import argparse
import logging
import os
import spacy
from collections import defaultdict
from nltk.stem import PorterStemmer
from overrides import overrides
from spacy.tokens import Token
from subprocess import Popen
from typing import Dict, List, Set, Tuple

from sacrerouge.commands import MetricSetupSubcommand
from sacrerouge.common import DATA_ROOT
from sacrerouge.data import MetricsDict
from sacrerouge.data.types import ReferenceType, SummaryType
from sacrerouge.metrics import Metric, ReferenceBasedMetric
from sacrerouge.metrics.decomposed_rouge.categorizers import DependencyCategorizer, DependencyVerbRelationsCategorizer, \
    NERCategorizer, NPChunkCategorizer, POSCategorizer, StopwordCategorizer
from sacrerouge.metrics.decomposed_rouge.categorizers.util import calculate_maximum_matching

logger = logging.getLogger(__name__)

# The text which should be used to match other tokens (could be the original token
# or the stem, for example)
Token.set_extension('matching_text', default=None)

# A flag which marks whether this token is eligible for matching
Token.set_extension('is_matchable', default=None)

# The token's index in the list of tokens generated from the document
Token.set_extension('index', default=None)

# A flag which marks if the token is part of a noun phrase
Token.set_extension('is_np', default=None)


@Metric.register('decomposed-rouge')
class DecomposedRouge(ReferenceBasedMetric):
    def __init__(self,
                 rouge_data_dir: str = f'{DATA_ROOT}/metrics/ROUGE-1.5.5/data',
                 remove_stopwords: bool = False,
                 use_porter_stemmer: bool = True,
                 pretokenized_text: bool = False):
        super().__init__()
        self.rouge_data_dir = rouge_data_dir
        self.remove_stopwords = remove_stopwords
        self.use_porter_stemmer = use_porter_stemmer
        self.pretokenized_text = pretokenized_text

        self.stemmer = PorterStemmer(PorterStemmer.ORIGINAL_ALGORITHM)
        self.stemmer_exceptions = self._load_stemmer_exceptions(rouge_data_dir)
        self.stopwords = self._load_stopwords(rouge_data_dir)
        self.nlp = spacy.load('en_core_web_sm')

        self.categorizers = [
            NERCategorizer('ner', [['PERSON'], ['NORP', 'ORG'], ['GPE', 'LOC']]),
            POSCategorizer('pos-adj', ['ADJ']),
            POSCategorizer('pos-verb', ['VERB']),
            POSCategorizer('pos-noun', ['NOUN']),
            POSCategorizer('pos-propn', ['PROPN']),
            POSCategorizer('pos-adv', ['ADV']),
            POSCategorizer('pos-num', ['NUM']),
            DependencyCategorizer('dep-root', ['ROOT']),
            DependencyCategorizer('dep-nsubj', ['nsubj']),
            DependencyCategorizer('dep-dobj', ['dobj']),
            NPChunkCategorizer(),
            DependencyVerbRelationsCategorizer('dep-verb+nsubj', ['nsubj']),
            DependencyVerbRelationsCategorizer('dep-verb+dobj', ['dobj']),
            DependencyVerbRelationsCategorizer('dep-verb+nsubj+dobj', ['nsubj', 'dobj']),
            StopwordCategorizer(self.rouge_data_dir)
        ]

    def _load_stemmer_exceptions(self, root: str) -> Dict[str, str]:
        exceptions = {}
        for filename in ['adj.exc', 'adv.exc', 'noun.exc', 'verb.exc']:
            file_path = os.path.join(root, 'WordNet-2.0-Exceptions', filename)
            with open(file_path, 'r') as f:
                for line in f:
                    # I think there is a bug in the original perl script
                    # to construct the exceptions database. Some of the lines
                    # have more than 2 words on them, but the script only
                    # maps the first to the second, ignoring the third.
                    columns = line.strip().split()
                    exceptions[columns[0]] = columns[1]
        return exceptions

    def _load_stopwords(self, root: str) -> Set[str]:
        file_path = os.path.join(root, 'smart_common_words.txt')
        return set(open(file_path, 'r').read().splitlines())

    def _preprocess_summary(self, summary: SummaryType) -> List[Token]:
        # The analysis doesn't pay attention to sentences, so we just wrap the string summary into
        # a list of sentences so the code runs the same regardless if the input is a string or a list of strings
        if isinstance(summary, str):
            summary = [summary]

        tokens = []
        for sentence in summary:
            if self.pretokenized_text:
                doc = self.nlp.tokenizer.tokens_from_list(sentence.split())
                self.nlp.tagger(doc)
                self.nlp.parser(doc)
                self.nlp.entity(doc)
            else:
                doc = self.nlp(sentence)

            for token in doc:
                text = token.text.lower()
                # There are some whitespace only tokens. I don't know where they come from
                if len(text.strip()) == 0:
                    continue

                is_matchable = True
                if self.remove_stopwords and text in self.stopwords:
                    is_matchable = False
                elif self.use_porter_stemmer and len(text) > 3:
                    if text in self.stemmer_exceptions:
                        text = self.stemmer_exceptions[text]
                    else:
                        text = self.stemmer.stem(text)

                token._.matching_text = text
                token._.is_matchable = is_matchable and not token.is_punct
                token._.index = len(tokens)
                tokens.append(token)

            for chunk in doc.noun_chunks:
                for token in chunk:
                    token._.is_np = True

        return tokens

    def _preprocess_all(self, summaries_list: List[List[str]]):
        tokens_lists = []
        for summaries in summaries_list:
            tokens_lists.append([])
            for summary in summaries:
                tokens_lists[-1].append(self._preprocess_summary(summary))
        return tokens_lists

    def _group_by_token(self, tokens: List[Token]) -> Dict[str, List[int]]:
        groups = defaultdict(list)
        for token in tokens:
            if token._.is_matchable:
                groups[token._.matching_text].append(token._.index)
        return groups

    def _get_matches(self,
                     summary_tokens: List[Token],
                     reference_tokens_list: List[List[Token]]) -> List[List[Tuple[int, int]]]:
        summary_groups = self._group_by_token(summary_tokens)
        reference_groups_list = [self._group_by_token(reference_tokens) for reference_tokens in reference_tokens_list]
        matches_list = []
        for reference_groups in reference_groups_list:
            matches_list.append([])
            for token in summary_groups.keys():
                if token in reference_groups:
                    summary_indices = summary_groups[token]
                    reference_indices = reference_groups[token]
                    for i in summary_indices:
                        for j in reference_indices:
                            matches_list[-1].append((i, j))
        return matches_list

    def _score_with_categorizers(self,
                                 summary_tokens: List[Token],
                                 reference_tokens_list: List[List[Token]],
                                 matches_list: List[List[Tuple[int, int]]]) -> MetricsDict:
        # This will be used to calculate the standard ROUGE score
        total_intersection = 0
        total_summary_tokens = 0
        total_reference_tokens = 0

        # This will hold the category-specific metrics
        category_metrics_list = [[] for _ in self.categorizers]

        for reference_tokens, matches in zip(reference_tokens_list, matches_list):
            weighted_matches = [(i, j, 1) for i, j in matches]
            intersection = calculate_maximum_matching(weighted_matches)
            total_intersection += intersection
            total_summary_tokens += len(summary_tokens)
            total_reference_tokens += len(reference_tokens)

            for i, categorizer in enumerate(self.categorizers):
                category_metrics_list[i].append(categorizer.select_matches(summary_tokens, reference_tokens, matches, intersection))

        # Calculate the ROUGE-1 score
        precision = total_intersection / total_summary_tokens * 100 if total_summary_tokens > 0 else 0
        recall = total_intersection / total_reference_tokens * 100 if total_reference_tokens > 0 else 0
        f1 = (2 * precision * recall) / (precision + recall) if (precision + recall) > 0 else 0
        metrics = MetricsDict({
            'decomposed-rouge': {
                'rouge-1': {
                    'precision': precision,
                    'recall': recall,
                    'f1': f1
                }
            }
        })

        # Average the category metrics together
        for category_metrics in category_metrics_list:
            metrics['decomposed-rouge'].update(sum(category_metrics) / len(category_metrics))

        return metrics

    def score_multi_all(self,
                        summaries_list: List[List[SummaryType]],
                        references_list: List[ReferenceType],
                        **kwargs) -> List[List[MetricsDict]]:
        num_summaries = sum(len(summaries) for summaries in summaries_list)
        count = 0

        metrics_list = []
        for summaries, references in zip(summaries_list, references_list):
            metrics_list.append([])
            reference_tokens_list = [self._preprocess_summary(reference) for reference in references]
            for summary in summaries:
                summary_tokens = self._preprocess_summary(summary)
                matches_list = self._get_matches(summary_tokens, reference_tokens_list)
                metrics_list[-1].append(self._score_with_categorizers(summary_tokens, reference_tokens_list, matches_list))

                count += 1
                if count % 100 == 0:
                    logger.info(f'Processed {count} / {num_summaries} summaries')
        logger.info('Finished processing summaries')

        return metrics_list


@MetricSetupSubcommand.register('decomposed-rouge')
class DecomposedRougeSetupSubcommand(MetricSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the Decomposed ROGUE metric'
        self.parser = parser.add_parser('decomposed-rouge', description=description, help=description)
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        logger.info(f'Downloading Spacy model en_core_web_sm')
        command = 'python -m spacy download en_core_web_sm'
        process = Popen(command, shell=True)
        process.communicate()
        if process.returncode != 0:
            print('DecomposedRouge setup failure')
        else:
            print('DecomposedRouge setup success')


import os
import pytest
import unittest
from collections import defaultdict
from typing import Dict, List

from sacrerouge.data import Metrics
from sacrerouge.io import JsonlReader

_task2_metrics_file_path = 'datasets/duc-tac/duc2004/v1.0/task2.metrics.jsonl'
_task5_metrics_file_path = 'datasets/duc-tac/duc2004/v1.0/task5.metrics.jsonl'


class TestDUC2004Metrics(unittest.TestCase):
    def _convert_to_dicts(self, metrics_list: List[Metrics]) -> Dict[str, Dict[str, Metrics]]:
        metrics_dicts = defaultdict(dict)
        for metrics in metrics_list:
            metrics_dicts[metrics.instance_id][metrics.summarizer_id] = metrics.metrics
        return metrics_dicts

    @pytest.mark.skipif(not os.path.exists(_task2_metrics_file_path), reason='DUC 2004 task 2 metrics file does not exist')
    def test_task2_system_level(self):
        summary_level_metrics = JsonlReader(_task2_metrics_file_path, Metrics).read()
        summary_level_metrics = self._convert_to_dicts(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # SEE/short.results.table2
        assert summary_level_metrics['d30001']['102']['peer_quality']['Q2'] == 1
        assert summary_level_metrics['d30005']['120']['coverage']['mean'] == 0.067
        assert summary_level_metrics['d30011']['A']['num_model_units'] == 11

    @pytest.mark.skipif(not os.path.exists(_task5_metrics_file_path), reason='DUC 2004 task 5 metrics file does not exist')
    def test_task5_system_level(self):
        summary_level_metrics = JsonlReader(_task5_metrics_file_path, Metrics).read()
        summary_level_metrics = self._convert_to_dicts(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # SEE/short.results.table5
        assert summary_level_metrics['d132']['109']['peer_quality']['Q2'] == 1
        assert summary_level_metrics['d147']['5']['coverage']['mean'] == 0.160
        assert summary_level_metrics['d164']['F']['num_model_units'] == 8

        # Responsiveness/results.table.R
        assert summary_level_metrics['d132']['24']['responsiveness'] == 1
        assert summary_level_metrics['d200']['86']['responsiveness'] == 0



import json
import os
import pytest
import subprocess
import unittest

from sacrerouge.common import TemporaryDirectory
from sacrerouge.common.testing.util import sacrerouge_command_exists

_metrics_file_path = 'datasets/duc-tac/tac2008/v1.0/task1.A-B.metrics.jsonl'


class TestStatisticalSignifianceTest(unittest.TestCase):
    def test_command_exists(self):
        assert sacrerouge_command_exists(['stat-sig-test'])

    @pytest.mark.skipif(not os.path.exists(_metrics_file_path), reason='TAC 2008 dataset does not exist')
    def test_correlate_reference(self):
        # We have to use the TAC 2008 data because the MultiLing data is too small for
        # the bootstrapping to work
        with TemporaryDirectory() as temp_dir:
            command = [
                'python', '-m', 'sacrerouge', 'stat-sig-test',
                '--metrics-jsonl-files', _metrics_file_path,
                '--dependent-metric', 'overall_responsiveness',
                '--metric-A', 'rouge-1_jk_precision',
                '--metric-B', 'rouge-2_jk_recall',
                '--summarizer-type', 'all',
                '--hypothesis-test', 'bootstrap-both',
                '--output-file', f'{temp_dir}/correlations.json',
                '--random-seed', '6',
                '--silent'
            ]
            subprocess.run(command, check=True)
            correlations = json.load(open(f'{temp_dir}/correlations.json', 'r'))

            assert correlations['dependent_metric'] == 'overall_responsiveness'
            assert correlations['metric_A'] == 'rouge-1_jk_precision'
            assert correlations['metric_B'] == 'rouge-2_jk_recall'
            assert correlations['summarizer_type'] == 'all'
            assert correlations['test_method'] == 'bootstrap-both'
            self.assertAlmostEqual(correlations['alpha'], 0.05, places=4)
            assert correlations['two_tailed'] is False
            assert correlations['H0'] == 'r(rouge-1_jk_precision, overall_responsiveness) <= r(rouge-2_jk_recall, overall_responsiveness)'
            assert correlations['H1'] == 'r(rouge-1_jk_precision, overall_responsiveness) > r(rouge-2_jk_recall, overall_responsiveness)'

            assert correlations['summary_level']['pearson']['pvalue'] == 0.829
            assert correlations['summary_level']['pearson']['is_significant'] is False
            assert correlations['summary_level']['spearman']['pvalue'] == 0.938
            assert correlations['summary_level']['spearman']['is_significant'] is False
            assert correlations['summary_level']['kendall']['pvalue'] == 0.929
            assert correlations['summary_level']['kendall']['is_significant'] is False

            assert correlations['system_level']['pearson']['pvalue'] == 0.603
            assert correlations['system_level']['pearson']['is_significant'] is False
            assert correlations['system_level']['spearman']['pvalue'] == 0.945
            assert correlations['system_level']['spearman']['is_significant'] is False
            assert correlations['system_level']['kendall']['pvalue'] == 0.977
            assert correlations['system_level']['kendall']['is_significant'] is False

            assert correlations['global']['pearson']['pvalue'] == 0.49
            assert correlations['global']['pearson']['is_significant'] is False
            assert correlations['global']['spearman']['pvalue'] == 0.831
            assert correlations['global']['spearman']['is_significant'] is False
            assert correlations['global']['kendall']['pvalue'] == 0.811
            assert correlations['global']['kendall']['is_significant'] is False


from sacrerouge.common.testing.metric_test_cases import ReferenceBasedMetricTestCase
from sacrerouge.common.testing.util import sacrerouge_command_exists
from sacrerouge.metrics import AutoSummENG


class TestAutoSummENG(ReferenceBasedMetricTestCase):
    def test_autosummeng(self):
        # This is a regression test, not necessarily a test for correctness
        metric = AutoSummENG()
        expected_output = [
            {'AutoSummENG': 0.137825, 'MeMoG': 0.15328, 'NPowER': 0.177641},
            {'AutoSummENG': 0.181275, 'MeMoG': 0.168514, 'NPowER': 0.205207},
            {'AutoSummENG': 0.24427, 'MeMoG': 0.269781, 'NPowER': 0.268968},
            {'AutoSummENG': 0.148213, 'MeMoG': 0.151705, 'NPowER': 0.182664},
            {'AutoSummENG': 0.14474, 'MeMoG': 0.178874, 'NPowER': 0.188991},
            {'AutoSummENG': 0.144389, 'MeMoG': 0.155941, 'NPowER': 0.181914},
            {'AutoSummENG': 0.210532, 'MeMoG': 0.207259, 'NPowER': 0.232329},
            {'AutoSummENG': 0.204209, 'MeMoG': 0.20244, 'NPowER': 0.227536},
            {'AutoSummENG': 0.136333, 'MeMoG': 0.127154, 'NPowER': 0.169001},
            {'AutoSummENG': 0.216562, 'MeMoG': 0.22591, 'NPowER': 0.241125},
            {'AutoSummENG': 0.242036, 'MeMoG': 0.251485, 'NPowER': 0.262288},
            {'AutoSummENG': 0.215287, 'MeMoG': 0.21972, 'NPowER': 0.23859}
        ]
        super().assert_expected_output(metric, expected_output)

    def test_autosummeng_order_invariant(self):
        metric = AutoSummENG()
        self.assert_order_invariant(metric)

    def test_command_exists(self):
        assert sacrerouge_command_exists(['autosummeng'])

    def test_setup_command_exists(self):
        assert sacrerouge_command_exists(['setup-metric', 'autosummeng'])


import json
import os
import shutil
import tarfile
from collections import Counter
from typing import Any, Dict, List, Tuple

from sacrerouge.common.util import download_file_from_google_drive, download_url_to_file
from sacrerouge.data import Metrics, MetricsDict
from sacrerouge.datasets.fabbri2020.pair_data import run_pair_data, parse_story_file
from sacrerouge.io import JsonlReader, JsonlWriter


def download_tars(output_dir: str, force: bool) -> Tuple[str, str]:
    # Downloads the "story" tarfiles from https://cs.nyu.edu/~kcho/DMQA/
    cnn_tar = f'{output_dir}/cnn_stories.tgz'
    dailymail_tar = f'{output_dir}/dailymail_stories.tgz'
    download_file_from_google_drive('0BwmD_VLjROrfTHk4NFg2SndKcjQ', cnn_tar, force=force)
    download_file_from_google_drive('0BwmD_VLjROrfM1BxdkxVaTY2bWs', dailymail_tar, force=force)
    return cnn_tar, dailymail_tar


def download_human_judgments(output_dir: str, force: bool) -> str:
    url = 'https://storage.googleapis.com/sfr-summarization-repo-research/model_annotations.aligned.jsonl'
    file_path = f'{output_dir}/model_annotations.aligned.jsonl'
    download_url_to_file(url, file_path, force)
    return file_path


def download_system_outputs(output_dir: str, force: bool) -> str:
    expanded_dir = f'{output_dir}/expanded/'
    for system_id in range(0, 24):
        url = f'https://storage.googleapis.com/sfr-summarization-repo-research/M{system_id}.tar.gz'
        file_path = f'{output_dir}/M{system_id}.tar.gz'
        download_url_to_file(url, file_path, force)

        model_expanded_dir = f'{expanded_dir}/M{system_id}'
        if os.path.exists(model_expanded_dir) and force:
            shutil.rmtree(model_expanded_dir)

        if not os.path.exists(model_expanded_dir):
            with tarfile.open(file_path, 'r') as tar:
                tar.extractall(expanded_dir)
    return expanded_dir


def load_judgments(file_path: str) -> Tuple[List[Dict[str, Any]], List[Dict[str, Any]], List[Metrics]]:
    summaries = []
    summaries_with_crowd = []
    metrics_list = []

    with JsonlReader(file_path) as f:
        for instance in f:
            instance_id = instance['id']
            summarizer_id = instance['model_id']
            filename = instance['filepath']
            summary = {'text': instance['decoded']}
            references = instance['references']
            expert_annotations = instance['expert_annotations']
            turker_annotations = instance['turker_annotations']
            document = instance['text']

            # The first reference is always the ground-truth
            # https://github.com/Yale-LILY/SummEval/issues/8
            assert len(references) == 11
            references[0] = {
                'summarizer_id': 'ground-truth',
                'summarizer_type': 'reference',
                'text': references[0]
            }
            for i in range(1, 11):
                references[i] = {
                    'summarizer_id': f'turker-{i}',
                    'summarizer_type': 'reference',
                    'text': references[i]
                }

            summaries.append({
                'instance_id': instance_id,
                'summarizer_id': summarizer_id,
                'summarizer_type': 'peer',
                'file_path': filename,
                'document': {'text': document},
                'summary': summary,
                'references': [references[0]]
            })
            summaries_with_crowd.append({
                'instance_id': instance_id,
                'summarizer_id': summarizer_id,
                'summarizer_type': 'peer',
                'file_path': filename,
                'document': {'text': document},
                'summary': summary,
                'references': references
            })

            expert_metrics = MetricsDict({
                'coherence': [annotation['coherence'] for annotation in expert_annotations],
                'consistency': [annotation['consistency'] for annotation in expert_annotations],
                'fluency': [annotation['fluency'] for annotation in expert_annotations],
                'relevance': [annotation['relevance'] for annotation in expert_annotations]
            })
            turker_metrics = MetricsDict({
                'coherence': [annotation['coherence'] for annotation in turker_annotations],
                'consistency': [annotation['consistency'] for annotation in turker_annotations],
                'fluency': [annotation['fluency'] for annotation in turker_annotations],
                'relevance': [annotation['relevance'] for annotation in turker_annotations]
            })
            both = MetricsDict({
                'expert': expert_metrics,
                'turker': turker_metrics
            })
            metrics = Metrics(instance_id, summarizer_id, 'peer', both)
            metrics_list.append(metrics)

    return summaries, summaries_with_crowd, metrics_list


def setup_documents(cnn_tar: str, dailymail_tar: str, output_dir: str, force: bool) -> None:
    cnn_dir = f'{output_dir}/raw/cnndm/cnn'
    dm_dir = f'{output_dir}/raw/cnndm/dailymail'
    for tar_path, target_path in [(cnn_tar, cnn_dir), (dailymail_tar, dm_dir)]:
        if os.path.exists(target_path) and force:
            print(f'Removing {target_path}')
            shutil.rmtree(target_path)

        if not os.path.exists(target_path):
            print(f'Untarring {tar_path} (it\'s pretty slow...)')
            with tarfile.open(tar_path, 'r') as tar:
                tar.extractall(f'{output_dir}/raw/cnndm')


def load_system_outputs(dirname: str, judged_summaries: List, documents_root: str) -> List:
    # Some of the output directories have multiple outputs, likely from different
    # versions of the models. This mapping marks which of those output files was
    # used in the annotations
    # https://github.com/Yale-LILY/SummEval/issues/8
    model_to_filename = {
        'M0': 'outputs.aligned.jsonl',
        'M1': 'outputs.aligned.jsonl',
        'M2': 'outputs.aligned.jsonl',
        'M5': 'outputs_rouge.aligned.jsonl',
        'M8': 'outputs_ptrgen+cov.aligned.jsonl',
        'M9': 'outputs_extabs+rl+rerank.aligned.jsonl',
        'M10': 'outputs_encdec.aligned.jsonl',
        'M11': 'outputs_novelty.aligned.jsonl',
        'M12': 'outputs.aligned.jsonl',
        'M13': 'outputs.aligned.jsonl',
        'M14': 'outputs.aligned.jsonl',
        'M15': 'outputs_coverage.aligned.jsonl',
        'M17': 'outputs_11B.aligned.jsonl',
        'M20': 'outputs_zeroshot.aligned.jsonl',
        'M22': 'outputs_cnndm.aligned.jsonl',
        'M23': 'outputs_c4_cnn_dailymail.aligned.jsonl',
    }

    document_cache = {}
    outputs = []

    # The model outputs can have duplicate IDs if the instances were mapped
    # to the same inputs. We only keep the first occurrence of any duplicate.
    # However, dm-test-2c37d44d03ce2e91310339d884d33ee5aabf9abc is duplicated and selected
    # to be judged. We want to ensure that the summary saved here is the one which
    # was judged, so we need to keep track of that information.
    seen = set()
    seen_and_judged = set()
    judged = {}
    for instance in judged_summaries:
        instance_id = instance['instance_id']
        summarizer_id = instance['summarizer_id']
        summary = instance['summary']['text']
        judged[(instance_id, summarizer_id)] = summary

    for summarizer_id, filename in sorted(model_to_filename.items()):
        file_path = f'{dirname}/{summarizer_id}/aligned/{filename}'
        print(f'Processing {file_path}')
        instances = JsonlReader(file_path).read()
        for instance in instances:
            instance_id = instance['id']

            # For some reason, there is an example with the filepath as the ID. The rest are not like this
            if instance_id == 'cnndm/dailymail/stories/cbed2a778a5e17d729c6e5ee5691b88710620dd7.story':
                instance_id = 'dm-test-cbed2a778a5e17d729c6e5ee5691b88710620dd7'

            # This json is messed up somehow, but you can see the decoded summary is empty
            if summarizer_id == 'M2' and instance_id == 'dm-test-9f270039c861e75ee2f01e4e2898a9ea04a96b26':
                summary = ''
            else:
                summary = instance['decoded']

            is_judged = (instance_id, summarizer_id) in judged
            if is_judged:
                judged_summary = judged[(instance_id, summarizer_id)]

                # This pair has a weird encoding which causes the summaries to not be equal.
                # It's not a duplicate instance, so we just take it
                if summarizer_id == 'M23' and instance_id == 'dm-test-2cf8c2d1d2ceb1980249f77e703f9039e63799d0':
                    summary = judged_summary

                if summary != judged_summary:
                    # We do not want this one
                    continue

            if (instance_id, summarizer_id) in seen:
                # We only want the first one
                continue
            seen.add((instance_id, summarizer_id))
            if is_judged:
                seen_and_judged.add((instance_id, summarizer_id))

            filepath = instance['filepath']
            if filepath not in document_cache:
                with open(f'{documents_root}/{filepath}', 'r') as f:
                    story_content = f.read()
                    document_cache[filepath] = parse_story_file(story_content)
            document = document_cache[filepath]

            reference = instance['reference']
            outputs.append({
                'instance_id': instance_id,
                'summarizer_id': summarizer_id,
                'summarizer_type': 'peer',
                'filepath': filepath,
                'document': {'text': document},
                'summary': {'text': summary},
                'reference': {'text': reference}
            })

    # Ensure that we saved all of the summaries that were judged
    assert seen_and_judged == set(judged.keys())

    return outputs


def ensure_no_duplicates(items: List) -> None:
    seen = set()
    for item in items:
        instance_id = item['instance_id']
        summarizer_id = item['summarizer_id']
        key = (instance_id, summarizer_id)
        assert key not in seen
        seen.add(key)


def sanity_check(judged_summaries: List, all_summaries: List, metrics_list: List[Metrics]) -> None:
    ensure_no_duplicates(judged_summaries)
    ensure_no_duplicates(all_summaries)

    judged_summaries = {
        (instance['instance_id'], instance['summarizer_id']): instance
        for instance in judged_summaries
    }
    all_summaries = {
        (instance['instance_id'], instance['summarizer_id']): instance
        for instance in all_summaries
    }
    metrics_dict = set([(instance.instance_id, instance.summarizer_id) for instance in metrics_list])

    # Ensure no duplicate metrics
    assert len(metrics_list) == len(metrics_list)

    # Ensure all judged are in all_summaries and metrics_dict
    assert len(judged_summaries) == len(metrics_dict)
    for key in judged_summaries.keys():
        assert key in all_summaries
        assert key in metrics_dict

        # Make sure the summaries are identical
        summary = all_summaries[key]['summary']['text']
        judged_summary = judged_summaries[key]['summary']['text']
        assert summary == judged_summary

        # Make sure the documents are identical
        document = all_summaries[key]['document']['text']
        judged_document = judged_summaries[key]['document']['text']
        assert document == judged_document

        # We do not check the references because they are different. The references
        # in all_summaries seem be normalized, whereas the summaries in judged_summaries
        # are not.
        # reference = all_summaries[key]['reference']['text']
        # judged_reference = judged_summaries[key]['references'][0]['text']
        # assert reference == judged_reference, key


def print_stats(summaries: List):
    summarizer_ids = Counter()
    instance_ids = Counter()

    for instance in summaries:
        instance_id = instance['instance_id']
        summarizer_id = instance['summarizer_id']
        summarizer_ids[summarizer_id] += 1
        instance_ids[instance_id] += 1

    print('Num instances per summarizer')
    print(json.dumps(summarizer_ids, indent=2))

    max_count = max(instance_ids.values())
    missing_summaries = {}
    for instance_id, count in instance_ids.items():
        if count != max_count:
            missing_summaries[instance_id] = count

    num_with_max = len(instance_ids) - len(missing_summaries)
    print('Num instances', len(instance_ids))
    print('Num instances with all systems having summaries', num_with_max)
    print('Instnaces with missing summaries')
    print(json.dumps(missing_summaries, indent=2))


def save_data(data: List[Any], file_path: str) -> None:
    with JsonlWriter(file_path) as out:
        for item in data:
            out.write(item)


def setup(output_dir: str, force: bool) -> None:
    cnn_tar, dailymail_tar = download_tars(f'{output_dir}/raw', force)

    # Download the expert and turker annotations
    download_human_judgments(f'{output_dir}/raw', force)

    # Download the system outputs
    model_output_dir = download_system_outputs(f'{output_dir}/raw/model-outputs', force)

    # Untar all of the documents
    setup_documents(cnn_tar, dailymail_tar, output_dir, force)

    # Pair together the summaries and input documents. The script will output the
    # results into "model_annotations.aligned.paired.jsonl"
    run_pair_data(data_annotations=f'{output_dir}/raw/model_annotations.aligned.jsonl',
                  model_outputs=model_output_dir,
                  story_files=f'{output_dir}/raw')

    summaries, summaries_with_crowd, metrics = load_judgments(f'{output_dir}/raw/model_annotations.aligned.paired.jsonl')
    save_data(summaries, f'{output_dir}/summaries.jsonl')
    save_data(summaries_with_crowd, f'{output_dir}/summaries-with-crowd.jsonl')
    save_data(metrics, f'{output_dir}/metrics.jsonl')

    system_outputs = load_system_outputs(model_output_dir, summaries, f'{output_dir}/raw')
    save_data(system_outputs, f'{output_dir}/all-summaries.jsonl.gz')

    sanity_check(summaries, system_outputs, metrics)
    print_stats(system_outputs)



import pytest

from sacrerouge.common.testing import FIXTURES_ROOT
from sacrerouge.common.testing.metric_test_cases import ReferenceBasedMetricTestCase
from sacrerouge.common.testing.util import sacrerouge_command_exists
from sacrerouge.io import JsonlReader
from sacrerouge.metrics import Meteor


class TestMeteor(ReferenceBasedMetricTestCase):
    def test_meteor(self):
        # This is a regression test, not necessarily a test for correctness
        metric = Meteor()
        expected_output = [
            {'METEOR': 0.1669124164573825},
            {'METEOR': 0.1948832291162732},
            {'METEOR': 0.20611797558912198},
            {'METEOR': 0.16514495147439157},
            {'METEOR': 0.17681198359839032},
            {'METEOR': 0.1711431947904745},
            {'METEOR': 0.21743329361081287},
            {'METEOR': 0.21450528119487908},
            {'METEOR': 0.16550404185166837},
            {'METEOR': 0.21164463762707278},
            {'METEOR': 0.20412130512572657},
            {'METEOR': 0.18243523574488876}
        ]
        super().assert_expected_output(metric, expected_output)

    def test_bewte_order_invariant(self):
        metric = Meteor()
        self.assert_order_invariant(metric)

    def test_chen2018(self):
        """
        Tests to ensure that Meteor returns the expected score on the
        Chen 2018 data subset. I ran Meteor on the full data (~11k examples)
        which takes too long to run for a unit test. After confirming the numbers
        are the same as what is reported in the paper, I ran the code on just
        the subset, and this test ensures those numbers are returned.
        """
        gold_file_path = f'{FIXTURES_ROOT}/data/chen2018/gold.jsonl'
        model_file_path = f'{FIXTURES_ROOT}/data/chen2018/model.jsonl'

        gold = JsonlReader(gold_file_path).read()
        model = JsonlReader(model_file_path).read()

        gold = [[summary['summary']] for summary in gold]
        model = [summary['summary'] for summary in model]

        meteor = Meteor()
        score, _ = meteor.evaluate(model, gold)
        assert score['METEOR'] == pytest.approx(0.1828372, abs=1e-7)

    def test_command_exists(self):
        assert sacrerouge_command_exists(['meteor'])

    def test_setup_command_exists(self):
        assert sacrerouge_command_exists(['setup-metric', 'meteor'])


import json
import os
import pytest
import subprocess
import unittest

from sacrerouge.common import TemporaryDirectory

_metrics_file_path = 'datasets/chaganty2018/metrics.jsonl'


class TestChaganty2018Correlation(unittest.TestCase):
    @pytest.mark.skipif(not os.path.exists(_metrics_file_path), reason='Chaganty 2018 metrics file does not exist')
    def test_correlation(self):
        with TemporaryDirectory() as temp_dir:
            command = [
                'python', '-m', 'sacrerouge', 'correlate',
                '--metrics-jsonl-files', _metrics_file_path,
                '--metrics', 'chaganty2018_overall', 'chaganty2018_rouge-1_recall',
                '--summarizer-type', 'peer',
                '--output-file', f'{temp_dir}/correlations.json',
                '--silent'
            ]
            subprocess.run(command, check=True)
            correlations = json.load(open(f'{temp_dir}/correlations.json', 'r'))
            # assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.910, 1e-2)
            # assert correlations['system_level']['pearson']['r'] == pytest.approx(0.778, 1e-2)
            # assert correlations['system_level']['num_summarizers'] == 4



import argparse
import json
import logging
import os
import string
from overrides import overrides
from subprocess import Popen, PIPE
from typing import Dict, List, Tuple

from sacrerouge.commands import MetricSetupSubcommand
from sacrerouge.common import DATA_ROOT, TemporaryDirectory
from sacrerouge.data import MetricsDict
from sacrerouge.data.types import ReferenceType
from sacrerouge.data.types import SummaryType
from sacrerouge.io import JsonlReader
from sacrerouge.metrics import Metric, ReferenceBasedMetric

logger = logging.getLogger(__name__)


@Metric.register('apes')
class APES(ReferenceBasedMetric):
    def __init__(self,
                 environment_name: str = 'apes',
                 apes_root: str = f'{DATA_ROOT}/metrics/apes',
                 verbose: bool = False):
        super().__init__()
        self.environment_name = environment_name
        if 'CONDA_INIT' not in os.environ:
            raise Exception('Environment variable "CONDA_INIT" must be set to the path to "conda.sh" since APES uses conda '
                            'to run the Python2.7 code.')

        self.apes_root = apes_root
        if not os.path.exists(self.apes_root):
            raise Exception(f'Path "{self.apes_root}" does not exist. Have you setup the metric?')

        self.verbose = verbose

    def _save_summaries(self,
                        output_file: str,
                        summaries_list: List[List[SummaryType]],
                        references_list: List[List[ReferenceType]]) -> Dict[str, List[str]]:
        instance_id_to_reference_ids = {}
        output_data = {}

        logger.info(f'Writing summaries to {output_file}')
        for i, (summaries, references) in enumerate(zip(summaries_list, references_list)):
            instance_id = str(i)
            output_data[instance_id] = {'man': {}, 'machine': {}}

            for j, summary in enumerate(summaries):
                summarizer_id = f'{instance_id}_{j}'
                if isinstance(summary, list):
                    text = ' '.join(summary)
                else:
                    text = summary
                output_data[instance_id]['machine'][summarizer_id] = text

            # We identify references with a letter
            assert len(references) <= 26
            reference_ids = []
            for j, reference in enumerate(references):
                reference_id = f'{instance_id}_{string.ascii_uppercase[j]}'
                reference_ids.append(reference_id)
                if isinstance(reference, list):
                    text = ' '.join(reference)
                else:
                    text = reference
                output_data[instance_id]['man'][reference_id] = text
            instance_id_to_reference_ids[instance_id] = reference_ids

        os.makedirs(os.path.dirname(output_file), exist_ok=True)
        with open(output_file, 'w') as out:
            out.write(json.dumps(output_data))

        return instance_id_to_reference_ids

    def _run_preprocess(self, input_file: str, output_file: str, metadata_file: str) -> Dict:
        logger.info('Running preprocessing')
        commands = [f'cd {self.apes_root}/APES-on-TAC2011']
        commands.append(f'source {os.environ["CONDA_INIT"]}')
        commands.append(f'conda activate {self.environment_name}')
        commands.append(f'python2.7 apes_on_tac2011.py --input-file {input_file} --output-file {output_file} --metadata-file {metadata_file}')
        command = ' && '.join(commands)

        logger.info(f'Running command: "{command}"')
        redirect = None if self.verbose else PIPE
        process = Popen(command, stdout=redirect, stderr=redirect, shell=True)
        process.communicate()

        metadata = json.load(open(metadata_file, 'r'))
        return metadata

    def _run_answer_questions(self, input_file: str, output_file: str) -> Dict[Tuple[str, str], Dict[str, float]]:
        logger.info('Running answering questions')
        commands = [f'cd {self.apes_root}/rc-cnn-dailymail']
        commands.append(f'source {os.environ["CONDA_INIT"]}')
        commands.append(f'conda activate {self.environment_name}')
        commands.append(f'python2.7 code/run_qa_model.py --input_file {input_file} --output_file {output_file} --train_path cnn_train.txt --dev_path cnn_dev.txt --glove_path glove.6B.100d.txt')
        command = ' && '.join(commands)

        logger.info(f'Running command: "{command}"')
        redirect = None if self.verbose else PIPE
        process = Popen(command, stdout=redirect, stderr=redirect, shell=True)
        process.communicate()

        ids_to_scores = {}
        with JsonlReader(output_file) as f:
            for data in f:
                summarizer_id = data['answering_doc']
                reference_id = data['questioning_doc']
                accuracy = data['acc']
                num_correct = data['num_correct']
                ids_to_scores[(summarizer_id, reference_id)] = {'accuracy': accuracy, 'num_correct': num_correct}
        return ids_to_scores

    def _get_metrics(self,
                     summaries_list: List[List[SummaryType]],
                     references_list: List[List[ReferenceType]],
                     instance_id_to_reference_ids: Dict[str, List[str]],
                     ids_to_scores: Dict[Tuple[str, str], Dict[str, float]],
                     metadata: Dict) -> List[List[MetricsDict]]:
        metrics_lists = []
        missing_entities = metadata['missing_entities']
        missing_questions = metadata['missing_questions']
        for i, (summaries, references) in enumerate(zip(summaries_list, references_list)):
            metrics_lists.append([])
            instance_id = str(i)
            reference_ids = instance_id_to_reference_ids[instance_id]

            num_missing_entities = len(missing_entities[instance_id]) if instance_id in missing_entities else 0
            if num_missing_entities > 0:
                logger.warning(f'{num_missing_entities} reference(s) for instance index {i} are missing entities. No questions were generated')

            num_missing_questions = len(missing_questions[instance_id]) if instance_id in missing_questions else 0
            if num_missing_questions > 0:
                logger.warning(f'{num_missing_questions} reference(s) for instance index {i} are missing questions')

            for j, summary in enumerate(summaries):
                summarizer_id = f'{i}_{j}'
                scores = []
                for reference_id in reference_ids:
                    if (summarizer_id, reference_id) in ids_to_scores:
                        scores.append(ids_to_scores[(summarizer_id, reference_id)])

                if len(scores) + num_missing_entities + num_missing_questions != len(reference_ids):
                    logger.warning(f'Summary {j} for instance {i} does not have a score for every reference. '
                                   f'#Score: {len(scores)}, #Missing Entities: {num_missing_entities}, #Missing Qs: {num_missing_questions} #Ref: {len(reference_ids)}')

                if len(scores) == 0:
                    # All references were missing entities
                    scores = [{'accuracy': 0.0, 'num_correct': 0}]

                final_accuracy = sum([s['accuracy'] for s in scores]) / len(scores)
                final_num_correct = sum([s['num_correct'] for s in scores])
                metrics_lists[-1].append(MetricsDict({'APES': {'accuracy': final_accuracy, 'num_correct': final_num_correct}}))
        return metrics_lists

    def score_multi_all(self,
                        summaries_list: List[List[SummaryType]],
                        references_list: List[List[ReferenceType]]) -> List[List[MetricsDict]]:
        with TemporaryDirectory() as temp_dir:
            temp_dir = os.path.abspath(temp_dir)

            summaries_file = f'{temp_dir}/summaries.json'
            questions_file = f'{temp_dir}/questions.jsonl'
            metadata_file = f'{temp_dir}/metadata.json'
            answers_file = f'{temp_dir}/answers.jsonl'

            instance_id_to_reference_ids = self._save_summaries(summaries_file, summaries_list, references_list)
            metadata = self._run_preprocess(summaries_file, questions_file, metadata_file)
            ids_to_scores = self._run_answer_questions(questions_file, answers_file)
            metrics_lists = self._get_metrics(summaries_list, references_list, instance_id_to_reference_ids, ids_to_scores, metadata)

            return metrics_lists


@MetricSetupSubcommand.register('apes')
class APESSetupSubcommand(MetricSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the APES metric'
        self.parser = parser.add_parser('apes', description=description, help=description)
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        commands = [
            f'mkdir -p {DATA_ROOT}/metrics/apes',
            f'cd {DATA_ROOT}/metrics/apes',
            f'git clone https://github.com/danieldeutsch/APES-on-TAC2011',
            f'git clone https://github.com/danieldeutsch/rc-cnn-dailymail',
            f'cd rc-cnn-dailymail',
            f'wget https://danieldeutsch.s3.amazonaws.com/sacrerouge/metrics/APES/cnn_train.txt.gz',
            f'wget https://danieldeutsch.s3.amazonaws.com/sacrerouge/metrics/APES/cnn_dev.txt.gz',
            f'zcat cnn_train.txt.gz > cnn_train.txt',
            f'zcat cnn_dev.txt.gz > cnn_dev.txt',
            f'rm cnn_train.txt.gz cnn_dev.txt.gz',
            f'wget http://nlp.stanford.edu/data/glove.6B.zip',
            f'unzip glove.6B.zip',
            f'rm glove.6B.50d.txt glove.6B.200d.txt glove.6B.300d.txt glove.6B.zip',
        ]
        command = ' && '.join(commands)

        process = Popen(command, shell=True)
        process.communicate()

        if process.returncode == 0:
            print('APES setup success')
        else:
            print('APES setup failure')



import bisect
import os
import re
from lxml import etree
from typing import List, Set, Tuple


def _find_closest_match(matches: List[int], index: int) -> int:
    differences = [abs(match - index) for match in matches]
    min_diff = min(differences)
    min_index = differences.index(min_diff)
    start = matches[min_index]
    return start, min_index


def _find_soft_matches(summary: str, text: str, start: int) -> Tuple[str, int]:
    # Many of the mismatches are due to weird whitespace issues, so we get rid
    # of the whitespace, find matches, and then remap to the summary

    # Maps from the character index in the summary without spaces to the
    # character index in the summary with spaces
    index_map = {}
    offset = 0
    for i, char in enumerate(summary):
        if char != ' ':
            index_map[offset] = i
            offset += 1

    edited_summary = summary.replace(' ', '')
    edited_text = text.replace(' ', '')

    regex = re.escape(edited_text)
    edited_starts = [match.start() for match in re.finditer(regex, edited_summary)]
    if len(edited_starts) == 0:
        return None, None
    starts = [index_map[index] for index in edited_starts]

    start, index = _find_closest_match(starts, start)
    end = index_map[edited_starts[index] + len(edited_text) - 1]
    found_text = summary[start:end + 1]

    return found_text, start


class Part(object):
    def __init__(self, text: str, start: int, end: int) -> None:
        self.text = text
        self.start = start
        self.end = end


class Contributor(object):
    def __init__(self, summary_index: int, label: str, parts: List[Part]) -> None:
        self.summary_index = summary_index
        self.label = label
        self.parts = parts


class ContributorAnnotation(object):
    def __init__(self, label: str, parts: List[Part]) -> None:
        self.label = label
        self.parts = parts


class SCU(object):
    def __init__(self, scu_id: int, label: str, contributors: List[Contributor]) -> None:
        self.scu_id = scu_id
        self.label = label
        self.contributors = contributors

    def get_weight(self) -> int:
        """Calculates the weight of the SCU"""
        # We don't keep this as a data member because then it would be serialized to the json. That's probably
        # an OK thing to do, but we have many already-created serialized pyramids, and adding that might break them.
        #
        # This should be just the length, but we take the unique number because there were many errors in the
        # DUC/TAC pyramid files which may make that assumption false.
        return len(set(contributor.summary_index for contributor in self.contributors))


class SCUAnnotation(object):
    def __init__(self, scu_id: int, label: str, contributors: List[ContributorAnnotation]) -> None:
        self.scu_id = scu_id
        self.label = label
        self.contributors = contributors


class Pyramid(object):
    def __init__(self,
                 instance_id: str,
                 summaries: List[str],
                 summarizer_ids: List[str],
                 scus: List[SCU]) -> None:
        self.instance_id = instance_id
        self.summaries = summaries
        self.summarizer_ids = summarizer_ids
        self.scus = scus

    def remove_summary(self, index: int) -> 'Pyramid':
        new_summaries = self.summaries[:index] + self.summaries[index + 1:]
        new_summarizer_ids = self.summarizer_ids[:index] + self.summarizer_ids[index + 1:]
        new_scus = []
        for scu in self.scus:
            contributors = []
            for contributor in scu.contributors:
                if contributor.summary_index != index:
                    # Potentially fix the summary index
                    summary_index = contributor.summary_index
                    if summary_index > index:
                        summary_index -= 1

                    contributors.append(Contributor(summary_index, contributor.label, contributor.parts))
            if len(contributors) > 0:
                new_scus.append(SCU(scu.scu_id, scu.label, contributors))
        return Pyramid(self.instance_id, new_summaries, new_summarizer_ids, new_scus)

    def get_annotation(self, index: int) -> 'PyramidAnnotation':
        scus = []
        for scu in self.scus:
            new_contributors = []
            for contributor in scu.contributors:
                if contributor.summary_index == index:
                    new_contributors.append(ContributorAnnotation(contributor.label, contributor.parts))
            if len(new_contributors) > 0:
                scus.append(SCUAnnotation(scu.scu_id, scu.label, new_contributors))
        return PyramidAnnotation(self.instance_id, self.summarizer_ids[index], 'reference', self.summaries[index], scus)

    def get_scu_id_set(self, index: int) -> Set[int]:
        scus = set()
        for i in range(len(self.summaries)):
            for scu in self.scus:
                for contributor in scu.contributors:
                    if contributor.summary_index == index:
                        scus.add(scu.scu_id)
        return scus

    @staticmethod
    def _get_summarizer_id(title_regex_match: str) -> str:
        # Remove any leading or trailing spaces or '-'
        start = 0
        while title_regex_match[start] in [' ', '-', '\n']:
            start += 1
        end = len(title_regex_match) - 1
        while title_regex_match[end] in [' ', '-', '\n']:
            end -= 1

        filename = title_regex_match[start:end + 1]
        columns = filename.split('.')
        return columns[-1]

    @staticmethod
    def _load_summaries(root, default_document_regex: str = None):
        text = '\n'.join([node.text if node.text else '' for node in root.xpath('./text/line')])

        # The pyramid files contain regular expressions that match the tags
        # which indicate a summary is about to start. Therefore, the summaries are
        # in between the tags
        nodes = list(root.xpath('./startDocumentRegEx'))
        if len(nodes) == 0:
            # If the document regex doesn't exist due to an error, we use the default
            assert default_document_regex is not None
            document_start_regex = re.compile(default_document_regex)
        else:
            document_start_regex = re.compile(nodes[0].text)

        # The starts and ends of the summaries, not the header tags
        starts, ends = [], []
        summarizer_ids = []
        for i, match in enumerate(document_start_regex.finditer(text)):
            if match.group(0).strip():
                summarizer_ids.append(Pyramid._get_summarizer_id(match.group(0)))
                start, end = match.span()
                # Find the first character of the summary
                while text[end].isspace():
                    end += 1
                starts.append(end)
                if i != 0:
                    ends.append(start)
        ends.append(len(text))

        summaries = [text[start:end].strip().replace('\n', ' ') for start, end in zip(starts, ends)]
        for summary in summaries:
            assert len(summary.strip()) > 0
        return summaries, starts, summarizer_ids

    @staticmethod
    def _load_scus(root,
                   summaries: List[str],
                   offsets: List[int]) -> List[SCU]:
        scus = []
        for node in root.xpath('scu'):
            label = node.get('label')
            # Some labels have endings like "(2.1)"
            label = re.sub(r' \(\d+\.\d+\)$', '', label)

            scu_id = int(node.get('uid'))
            contributors = []

            for contrib_node in node.xpath('./contributor'):
                contrib_label = contrib_node.get('label')
                summary_indices = []
                parts = []
                for part_node in contrib_node.xpath('./part'):
                    text = part_node.get('label')
                    start = int(part_node.get('start'))
                    end = int(part_node.get('end'))

                    summary_index = bisect.bisect_right(offsets, start) - 1
                    assert summary_index >= 0
                    summary_indices.append(summary_index)

                    # Make sure the label of the part is identical to the text from the summary
                    start -= offsets[summary_index]
                    end -= offsets[summary_index]
                    if text != summaries[summary_index][start:end]:
                        found_text, found_start = _find_soft_matches(summaries[summary_index], text, start)
                        if found_text is None:
                            # Sometimes there's a &quot; character which messes it up. Try again
                            text = re.sub('&quot;', '"', text)
                            found_text, found_start = _find_soft_matches(summaries[summary_index], text, start)
                            if found_text is None:
                                print(f'Cannot find text: "{text}". Skipping part')
                                continue
                        start = found_start
                        end = start + len(found_text)
                        text = found_text

                    parts.append(Part(text, start, end))

                if len(parts) == 0:
                    print(f'SCU {scu_id} contributor "{contrib_label}" has 0 valid parts. Skipping')
                    continue

                if len(set(summary_indices)) > 1:
                    print(f'SCU {scu_id} contributor "{contrib_label}" comes from multiple summaries. Skipping contributor')
                    continue
                else:
                    summary_index = summary_indices[0]

                contributors.append(Contributor(summary_index, contrib_label, parts))

            if len(contributors) == 0:
                # This could happen if all of the contributors are skipped due to errors
                print(f'SCU {scu_id} has 0 valid contributors. Skipping SCU')
                continue

            contributors_ok = len(contributors) == len(set([contributor.summary_index for contributor in contributors]))
            if not contributors_ok:
                print(f'SCU {scu_id} has multiple contributors with identical summary indices. Skipping SCU')
                continue

            scus.append(SCU(scu_id, label, contributors))
            assert len(contributors) <= len(offsets)

        return scus

    @staticmethod
    def from_xml(instance_id: str,
                 file_path_or_xml: str,
                 default_document_regex: str = None,
                 is_combined_file: bool = False) -> 'Pyramid':
        if os.path.exists(file_path_or_xml):
            xml = open(file_path_or_xml, 'r').read()
        else:
            xml = file_path_or_xml

        root = etree.fromstring(xml)
        if is_combined_file:
            root = root.xpath('./pyramid')[0]

        summaries, offsets, summarizer_ids = Pyramid._load_summaries(root, default_document_regex=default_document_regex)
        scus = Pyramid._load_scus(root, summaries, offsets)
        return Pyramid(instance_id, summaries, summarizer_ids, scus)


class PyramidAnnotation(object):
    def __init__(self,
                 instance_id: str,
                 summarizer_id: str,
                 summarizer_type: str,
                 summary: str,
                 scus: List[SCUAnnotation]) -> None:
        self.instance_id = instance_id
        self.summarizer_id = summarizer_id
        self.summarizer_type = summarizer_type
        self.summary = summary
        self.scus = scus

    def get_scu_id_set(self) -> Set[int]:
        return set([scu.scu_id for scu in self.scus])

    @staticmethod
    def _load_summary(root) -> str:
        lines = []
        for node in root.xpath('./annotation/text/line'):
            if node.text:
                lines.append(node.text.strip())
        summary = ' '.join(lines)
        summary = re.sub(r'\s+', ' ', summary).strip()
        return summary

    @staticmethod
    def _find_exact_matches(summary: str, text: str) -> List[int]:
        # Find all exact of the text in the summary. Return the list of offsets
        regex = re.escape(text)
        return [match.start() for match in re.finditer(regex, summary)]

    @staticmethod
    def _load_scus(root, summary: str, pyramid: Pyramid) -> List[SCUAnnotation]:
        # Because of parsing errors for the original pyramid, there may be some SCUs
        # which are no longer valid (because they were dropped in the parsing). We
        # will skip those
        known_scu_ids = set([scu.scu_id for scu in pyramid.scus])

        scus = []
        # Iterate over all peerscu nodes with a contributor child, others
        # are not matches
        for node in root.xpath('./annotation/peerscu[contributor]'):
            label = node.get('label')
            label = re.sub(r'^\(\d+\) ', '', label)
            label = re.sub(r' \(\d+\.\d+\)$', '', label)

            scu_id = int(node.get('uid'))
            if scu_id not in known_scu_ids:
                continue

            # SCU 0 is a catchall for non-matches
            if scu_id == 0:
                continue
            contributors = []
            for contributor_node in node.xpath('./contributor'):
                contrib_label = contributor_node.get('label')
                parts = []
                for part_node in contributor_node.xpath('./part'):
                    text = part_node.get('label')
                    text = re.sub(r'\s+', ' ', text)
                    start = int(part_node.get('start'))

                    matches = PyramidAnnotation._find_exact_matches(summary, text)
                    if len(matches) > 0:
                        # Find the match which is closest to "start"
                        found_text = text
                        start, _ = _find_closest_match(matches, start)
                    elif len(matches) == 0:
                        found_text, start = _find_soft_matches(summary, text, start)
                        if found_text is None:
                            print(f'Could not find part "{text}" for SCU {scu_id}. Skipping')
                            continue

                    end = start + len(found_text)
                    assert summary[start:end] == found_text
                    parts.append(Part(found_text, start, end))

                if len(parts) == 0:
                    print(f'No parts for contributor of SCU {scu_id}. Skipping')
                else:
                    contributors.append(ContributorAnnotation(contrib_label, parts))

            if len(contributors) == 0:
                print(f'No contributors for SCU {scu_id}. Skipping')
            else:
                scus.append(SCUAnnotation(scu_id, label, contributors))

        return scus

    @staticmethod
    def from_xml(instance_id: str,
                 summarizer_id: str,
                 summarizer_type: str,
                 file_path_or_xml: str,
                 pyramid: Pyramid) -> 'PyramidAnnotation':
        if os.path.exists(file_path_or_xml):
            xml = open(file_path_or_xml, 'r').read()
        else:
            xml = file_path_or_xml

        root = etree.fromstring(xml)
        summary = PyramidAnnotation._load_summary(root)
        if len(summary) == 0:
            # For some reason, there are blank summaries in some files
            # (see TAC 2008, D0805-A.M.100.A.5)
            print(f'Summary for instance {instance_id} and summarizer {summarizer_id} is `None`')
            return None
        scus = PyramidAnnotation._load_scus(root, summary, pyramid)
        return PyramidAnnotation(instance_id, summarizer_id, summarizer_type, summary, scus)



import argparse
from overrides import overrides

from sacrerouge.datasets.multiling.multiling2017 import sds, sds_metrics
from sacrerouge.commands import DatasetSetupSubcommand
from sacrerouge.common.util import download_file_from_google_drive


@DatasetSetupSubcommand.register('multiling2017')
class MultiLing2017Subcommand(DatasetSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the MultiLing 2017 dataset'
        self.parser = parser.add_parser('multiling2017', description=description, help=description)
        self.parser.add_argument(
            'output_dir',
            type=str,
            help='The directory where the data should be saved'
        )
        self.parser.set_defaults(subfunc=self.run)

    def _notify_about_license(self):
        print('This dataset is distributed by MultiLing 2017. If you use this dataset, please refer to their '
              'website (http://multiling.iit.demokritos.gr/pages/view/1616/multiling-2017)'
              'for instructions on proper attribution.')
        print()
        response = input('Continue? [y/n]: ')
        if response.lower() != 'y':
            exit()

    @overrides
    def run(self, args):
        self._notify_about_license()

        # Link provided by John Conroy
        data_path = f'{args.output_dir}/2017_test_data.tgz'
        download_file_from_google_drive('1dQfEYzJokm0es3xFHJG1J3crBAYU5zTp', data_path)

        eval_path = f'{args.output_dir}/EvaluationML2017.tgz'
        download_file_from_google_drive('1pK7Df5gum5mwC0zYie5mCjqDdqLLmM1j', eval_path)

        sds.setup(data_path, eval_path, f'{args.output_dir}/sds')
        sds_metrics.setup(eval_path, f'{args.output_dir}/sds')



import argparse
import tarfile
from collections import defaultdict

from sacrerouge.data import Metrics, MetricsDict
from sacrerouge.io import JsonlWriter


def parse_filename(filename: str):
    parts = filename.split('.')
    assert len(parts) == 5
    instance_id = parts[0].split('-')[0].lower()
    group = parts[0].split('-')[1]
    summarizer_id = parts[4]
    return instance_id, group, summarizer_id


def load_summaries(eval_tar: str):
    summaries = defaultdict(lambda: defaultdict(dict))
    with tarfile.open(eval_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile() and (member.name.startswith('UpdateSumm08_eval/ROUGE/models/') or member.name.startswith('UpdateSumm08_eval/ROUGE/peers/')):
                path = member.name.split('/')
                filename = path[-1]
                instance_id, group, summarizer_id = parse_filename(filename)

                if summarizer_id.isalpha():
                    summarizer_type = 'reference'
                else:
                    summarizer_type = 'peer'

                sentences = tar.extractfile(member).read().decode(errors='replace').splitlines()
                sentences = list(filter(None, map(lambda sentence: sentence.strip(), sentences)))
                summary = {
                    'summarizer_id': summarizer_id,
                    'summarizer_type': summarizer_type,
                    'text': sentences
                }
                summaries[instance_id][summarizer_id][group] = summary
    return summaries


def load_manual_judgments(eval_tar: str, metrics):
    with tarfile.open(eval_tar, 'r') as tar:
        lines = tar.extractfile('UpdateSumm08_eval/manual/manual.model').read().decode().splitlines()
        for line in lines:
            columns = line.split()
            instance_id = columns[0].split('-')[0].lower()
            group = columns[0].split('-')[1]
            summarizer_id = columns[1]
            metrics[instance_id][group][summarizer_id] = {
                'num_scus_jk': int(columns[2]),
                'modified_pyramid_score_jk': float(columns[4]),
                'linguistic_quality': int(columns[5]),
                'overall_responsiveness': int(columns[6])
            }

        lines = tar.extractfile('UpdateSumm08_eval/manual/manual.peer').read().decode().splitlines()
        for line in lines:
            columns = line.split()
            instance_id = columns[0].split('-')[0].lower()
            group = columns[0].split('-')[1]
            summarizer_id = columns[1]
            metrics[instance_id][group][summarizer_id] = {
                'modified_pyramid_score': float(columns[2]),
                'num_scus': int(columns[3]),
                'num_repetitions': int(columns[4]),
                'modified_pyramid_score_jk': float(columns[6]),
                'linguistic_quality': int(columns[7]),
                'overall_responsiveness': int(columns[8])
            }


def load_rouge_output(eval_tar: str, file_path: str, metrics):
    with tarfile.open(eval_tar, 'r') as tar:
        lines = tar.extractfile(file_path).read().decode().splitlines()
        for line in lines:
            columns = line.split()
            if len(columns) == 7 and columns[2] == 'Eval':
                summarizer_id = columns[0]
                rouge_metric = columns[1].lower()
                instance_id, group, summarizer_id = parse_filename(columns[3])
                recall = float(columns[4][2:]) * 100
                precision = float(columns[5][2:]) * 100
                f1 = float(columns[6][2:]) * 100
                metrics[instance_id][group][summarizer_id][rouge_metric] = {
                    'recall': recall,
                    'precision': precision,
                    'f1': f1
                }


def load_rouge_jk_output(eval_tar: str, file_path: str, metrics):
    jk_metrics = defaultdict(lambda: defaultdict(lambda: defaultdict(lambda: defaultdict(lambda: defaultdict(list)))))
    with tarfile.open(eval_tar, 'r') as tar:
        lines = tar.extractfile(file_path).read().decode().splitlines()
        for line in lines:
            columns = line.split()
            if len(columns) == 7 and columns[2] == 'Eval':
                summarizer_id = columns[0]
                rouge_metric = columns[1].lower() + '_jk'
                instance_id, group, summarizer_id = parse_filename(columns[3])

                recall = float(columns[4][2:]) * 100
                precision = float(columns[5][2:]) * 100
                f1 = float(columns[6][2:]) * 100
                jk_metrics[instance_id][group][summarizer_id][rouge_metric]['recall'].append(recall)
                jk_metrics[instance_id][group][summarizer_id][rouge_metric]['precision'].append(precision)
                jk_metrics[instance_id][group][summarizer_id][rouge_metric]['f1'].append(f1)

        for instance_id in jk_metrics.keys():
            for group in ['A', 'B']:
                for summarizer_id in jk_metrics[instance_id][group].keys():
                    for rouge_metric in jk_metrics[instance_id][group][summarizer_id].keys():
                        recalls = jk_metrics[instance_id][group][summarizer_id][rouge_metric]['recall']
                        precisions = jk_metrics[instance_id][group][summarizer_id][rouge_metric]['precision']
                        f1s = jk_metrics[instance_id][group][summarizer_id][rouge_metric]['f1']
                        metrics[instance_id][group][summarizer_id][rouge_metric] = {
                            'recall': sum(recalls) / len(recalls),
                            'precision': sum(precisions) / len(precisions),
                            'f1': sum(f1s) / len(f1s)
                        }


def get_references(summaries, instance_id, summarizer_id, group):
    summarizer_ids = list(summaries[instance_id].keys())
    reference_ids = list(filter(lambda sid: sid.isalpha(), summarizer_ids))

    references = []
    for reference_id in reference_ids:
        if summarizer_id == reference_id:
            continue
        references.append(summaries[instance_id][reference_id][group])
    return references


def save_summaries_and_metrics(summaries, metrics, output_dir: str):
    with JsonlWriter(f'{output_dir}/task1.A-B.summaries.jsonl') as out_summaries_A_B:
        with JsonlWriter(f'{output_dir}/task1.A.summaries.jsonl') as out_summaries_A:
            with JsonlWriter(f'{output_dir}/task1.B.summaries.jsonl') as out_summaries_B:
                with JsonlWriter(f'{output_dir}/task1.A-B.metrics.jsonl') as out_metrics_A_B:
                    with JsonlWriter(f'{output_dir}/task1.A.metrics.jsonl') as out_metrics_A:
                        with JsonlWriter(f'{output_dir}/task1.B.metrics.jsonl') as out_metrics_B:
                            for instance_id in sorted(summaries.keys()):
                                for summarizer_id in sorted(summaries[instance_id].keys()):
                                    summary_A = summaries[instance_id][summarizer_id]['A']
                                    summary_B = summaries[instance_id][summarizer_id]['B']

                                    references_A = get_references(summaries, instance_id, summarizer_id, 'A')
                                    references_B = get_references(summaries, instance_id, summarizer_id, 'B')

                                    metrics_A = metrics[instance_id]['A'][summarizer_id]
                                    metrics_B = metrics[instance_id]['B'][summarizer_id]

                                    summary_instance_A = {
                                        'instance_id': f'{instance_id}-A',
                                        'summarizer_id': summarizer_id,
                                        'summarizer_type': summary_A['summarizer_type'],
                                        'summary': summary_A,
                                        'references': references_A,
                                    }
                                    summary_instance_B = {
                                        'instance_id': f'{instance_id}-B',
                                        'summarizer_id': summarizer_id,
                                        'summarizer_type': summary_B['summarizer_type'],
                                        'summary': summary_B,
                                        'references': references_B,
                                        'metrics': metrics_B
                                    }
                                    metric_instance_A = Metrics(f'{instance_id}-A', summarizer_id, summary_A['summarizer_type'], metrics_A)
                                    metric_instance_B = Metrics(f'{instance_id}-B', summarizer_id, summary_B['summarizer_type'], metrics_B)

                                    out_summaries_A_B.write(summary_instance_A)
                                    out_summaries_A_B.write(summary_instance_B)
                                    out_summaries_A.write(summary_instance_A)
                                    out_summaries_B.write(summary_instance_B)

                                    out_metrics_A_B.write(metric_instance_A)
                                    out_metrics_A_B.write(metric_instance_B)
                                    out_metrics_A.write(metric_instance_A)
                                    out_metrics_B.write(metric_instance_B)


def setup(data_dir: str, output_dir: str):
    eval_tar = f'{data_dir}/scrapes/tac.nist.gov/protected/past-aquaint2/2008/UpdateSumm08_eval.tar.gz'
    main(eval_tar, output_dir)


def main(eval_tar, output_dir):
    summaries = load_summaries(eval_tar)

    metrics = defaultdict(lambda: defaultdict(lambda: defaultdict(MetricsDict)))
    load_manual_judgments(eval_tar, metrics)
    load_rouge_output(eval_tar, 'UpdateSumm08_eval/ROUGE/rouge.m.out', metrics)
    load_rouge_jk_output(eval_tar, 'UpdateSumm08_eval/ROUGE/rougejk.m.out', metrics)
    load_rouge_output(eval_tar, 'UpdateSumm08_eval/BE/simple.m.hm.out', metrics)
    load_rouge_jk_output(eval_tar, 'UpdateSumm08_eval/BE/simplejk.m.hm.out', metrics)

    save_summaries_and_metrics(summaries, metrics, output_dir)


if __name__ == '__main__':
    argp = argparse.ArgumentParser()
    argp.add_argument('eval_tar')
    argp.add_argument('output_dir')
    args = argp.parse_args()

    main(args.eval_tar, args.output_dir)



import argparse
from overrides import overrides
from typing import Type

from sacrerouge.commands import Subcommand
from sacrerouge.commands.evaluate import add_evaluate_arguments, evaluate_instances, save_evaluation_results
from sacrerouge.commands.score import add_score_arguments, save_score_results, score_instances
from sacrerouge.common import Registrable
from sacrerouge.common.arguments import add_metric_arguments, get_dataset_reader_from_argument, get_metric_from_arguments
from sacrerouge.common.logging import prepare_global_logging
from sacrerouge.common.util import import_module_and_submodules
from sacrerouge.metrics import Metric


def add_metric_subcommands(subparsers: argparse._SubParsersAction) -> None:
    """Adds a MetricSubcommand for every registered metric."""
    for name, (metric, _) in Registrable._registry[Metric].items():
        command = MetricSubcommand(name, metric)
        command.add_subparser(subparsers)


def add_dataset_reader_arguments(parser: argparse.ArgumentParser) -> None:
    parser.add_argument(
        '--dataset-reader',
        type=str,
        required=True,
        help='The name or the parameters as a serialized json for the dataset reader'
    )
    parser.add_argument(
        '--input-files',
        nargs='+',
        required=True,
        help='The input files to be passed to the dataset reader'
    )


class MetricSubcommand(Subcommand):
    def __init__(self, name: str, metric_type: Type) -> None:
        super().__init__()
        self.name = name
        self.metric_type = metric_type

    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = f'Run "evaluate" or "score" with the "{self.name}" metric.'
        self.parser = parser.add_parser(self.name, description=description, help=description)
        subparsers = self.parser.add_subparsers()

        description = f'Run "evaluate" with the "{self.name}" metric.'
        self.evaluate_parser = subparsers.add_parser('evaluate', description=description, help=description)
        add_evaluate_arguments(self.evaluate_parser, False)
        add_metric_arguments(self.evaluate_parser, self.metric_type)
        add_dataset_reader_arguments(self.evaluate_parser)
        self.evaluate_parser.set_defaults(func=self.run_evaluate)

        description = f'Run "score" with the "{self.name}" metric.'
        self.score_parser = subparsers.add_parser('score', description=description, help=description)
        add_score_arguments(self.score_parser, False)
        add_metric_arguments(self.score_parser, self.metric_type)
        add_dataset_reader_arguments(self.score_parser)
        self.score_parser.set_defaults(func=self.run_score)

    def run_evaluate(self, args: argparse.Namespace) -> None:
        prepare_global_logging(file_path=args.log_file, silent=args.silent)

        import_module_and_submodules('sacrerouge')
        include_packages = args.include_packages or []
        for package in include_packages:
            import_module_and_submodules(package)

        dataset_reader = get_dataset_reader_from_argument(args.dataset_reader)
        metric = get_metric_from_arguments(self.metric_type, args)
        input_files = args.input_files

        instances = dataset_reader.read(*input_files)
        macro, micro_list = evaluate_instances(instances, [metric])

        save_evaluation_results(macro, micro_list, args.macro_output_json, args.micro_output_jsonl, args.silent)

    def run_score(self, args: argparse.Namespace) -> None:
        prepare_global_logging(file_path=args.log_file, silent=args.silent)

        import_module_and_submodules('sacrerouge')
        include_packages = args.include_packages or []
        for package in include_packages:
            import_module_and_submodules(package)

        dataset_reader = get_dataset_reader_from_argument(args.dataset_reader)
        metric = get_metric_from_arguments(self.metric_type, args)
        input_files = args.input_files

        instances = dataset_reader.read(*input_files)
        metrics_dicts = score_instances(instances, [metric], args.disable_peer_jackknifing)

        save_score_results(metrics_dicts, args.output_jsonl, args.silent)


from overrides import overrides
from typing import List

from sacrerouge.data.fields import Fields, PyramidField, ReferencesField
from sacrerouge.data.pyramid import Pyramid


class Jackknifer(object):
    """
    A `Jackknifer` is responsible for taking a `Fields` object that requires
    jackknifing and converting it to a `List[Fields]` of jackknifed instances such
    that each item in the list is one of the jackknifing evaluations.
    """

    def get_jackknifing_fields_list(self, fields: Fields) -> List[Fields]:
        """
        Creates the list of jackknifed fields from the input `fields`. The input `fields`
        contains all of the fields which will be used to evaluate this particular instance.
        This method needs to return a list of new `Fields` that will each be evaluated.

        For example, if the references need to be jackknived for the input:

            {"summary": summary, "references": [reference1, reference2, reference3]}

        The method would return a list of three instances:

            [
                {"summary": summary, "references": [reference1, reference2]},
                {"summary": summary, "references": [reference1, reference3]},
                {"summary": summary, "references": [reference2, reference3]}
            ]

        If the `fields` cannot be jackknived (for example, if the above input only provided
        one reference), `None` should be returned.

        Args:
            fields (Fields): The fields that should be jackknived.

        Returns (List[Fields]):
            The jackknifed list of fields
        """
        raise NotImplementedError


class ReferencesJackknifer(Jackknifer):
    """
    Jackknives the "references" field of type `ReferencesField`.
    """

    @overrides
    def get_jackknifing_fields_list(self, fields: Fields) -> List[Fields]:
        references_field = fields['references']
        if len(references_field.references) == 1:
            # No jackknifing can be done, return `None` to indicate it cannot be done
            return None

        jk_fields_list = []
        for i in range(len(references_field.references)):
            # Copy the original fields and replace the references
            jk_fields = Fields(fields)
            jk_fields['references'] = ReferencesField(references_field.references[:i] + references_field.references[i + 1:])
            jk_fields_list.append(jk_fields)
        return jk_fields_list


class PyramidJackknifer(Jackknifer):
    """
    Jackknives the "pyramid" field of type `PyramidField`
    """
    @overrides
    def get_jackknifing_fields_list(self, fields: Fields) -> List[Fields]:
        pyramid: Pyramid = fields['pyramid'].pyramid
        if len(pyramid.summarizer_ids) == 1:
            # No jackknifing can be done, return `None`
            return None

        jk_fields_list = []
        for i in range(len(pyramid.summarizer_ids)):
            # Copy the original fields and replace the pyramid
            jk_fields = Fields(fields)
            jk_fields['pyramid'] = PyramidField(pyramid.remove_summary(i))
            jk_fields_list.append(jk_fields)
        return jk_fields_list



import os
import pytest

from sacrerouge.common.testing.metric_test_cases import ReferenceBasedMetricTestCase
from sacrerouge.common.testing.util import sacrerouge_command_exists
from sacrerouge.metrics import APES


@pytest.mark.skipif('APES_ENV' not in os.environ, reason='PyrEval python environment environment variable not set')
class TestQAEval(ReferenceBasedMetricTestCase):
    def test_apes(self):
        # This is a regression test, not necessarily a test for correctness
        metric = APES(environment_name=os.environ['APES_ENV'], verbose=True)
        expected_output = [
            {'APES': {'accuracy': 10.256410256410257, 'num_correct': 4}},
            {'APES': {'accuracy': 30.448717948717952, 'num_correct': 10}},
            {'APES': {'accuracy': 21.634615384615387, 'num_correct': 5}},
            {'APES': {'accuracy': 23.076923076923077, 'num_correct': 6}},
            {'APES': {'accuracy': 36.11111111111111, 'num_correct': 14}},
            {'APES': {'accuracy': 24.074074074074076, 'num_correct': 10}},
            {'APES': {'accuracy': 34.72222222222222, 'num_correct': 10}},
            {'APES': {'accuracy': 36.111111111111114, 'num_correct': 10}},
            {'APES': {'accuracy': 35.90203106332139, 'num_correct': 28}},
            {'APES': {'accuracy': 40.47422111938241, 'num_correct': 31}},
            {'APES': {'accuracy': 41.308243727598565, 'num_correct': 22}},
            {'APES': {'accuracy': 32.81637717121588, 'num_correct': 19}}
        ]
        super().assert_expected_output(metric, expected_output)

    def test_apes_order_invariant(self):
        metric = APES(environment_name=os.environ['APES_ENV'])
        self.assert_order_invariant(metric)

    def test_command_exists(self):
        assert sacrerouge_command_exists(['apes'])

    def test_setup_command_exists(self):
        assert sacrerouge_command_exists(['setup-metric', 'apes'])


import binascii


def is_gz_file(file_path: str):
    """
    Tests to see if a file is gzipped or not. This was taken from
    https://stackoverflow.com/questions/3703276/how-to-tell-if-a-file-is-gzip-compressed

    Returns
    -------
    True if it is gzipped, False otherwise.
    """
    with open(file_path, 'rb') as f:
        return binascii.hexlify(f.read(2)) == b'1f8b'



import zipfile
from collections import defaultdict

from sacrerouge.io import JsonlWriter
from sacrerouge.datasets.multiling.util import LANGUAGE_CODES


def load_documents(documents_zip: str):
    documents = defaultdict(lambda: defaultdict(list))

    with zipfile.ZipFile(documents_zip) as zip:
        for file_path in zip.namelist():
            if file_path.startswith('SourceTextsV2b/M'):
                filename = file_path[15:]
                # The filenames are like "M1047.english" where the cluster is "M104", it is the 7th document, and an English document
                instance_id, language = filename.split('.')
                instance_id = instance_id[:-1]
                language_code = LANGUAGE_CODES[language]

                # Contents are the plain text of the document. Blank lines seem to separate paragraphs.
                # No sentence breaking done
                contents = zip.open(file_path, 'r').read().decode()
                text = list(filter(None, [line.strip() for line in contents.splitlines()]))

                document = {
                    'filename': filename,
                    'text': text
                }
                documents[language_code][instance_id].append(document)
    return documents


def load_summaries(summaries_zip: str):
    summaries = defaultdict(lambda: defaultdict(list))
    with zipfile.ZipFile(summaries_zip) as zip:
        for file_path in zip.namelist():
            if file_path.endswith('.250'):
                path = file_path.split('/')
                language_code = LANGUAGE_CODES[path[1]]
                parts = path[2].split('.')
                instance_id = parts[0]
                annotator = parts[1]

                # The summaries don't seem to have any structure, so
                # just read in the full text.
                contents = zip.open(file_path, 'r').read().decode()
                text = list(filter(None, [line.strip() for line in contents.splitlines()]))
                summaries[language_code][instance_id].append({
                    'annotator': annotator,
                    'text': text
                })
    return summaries


def save_data(documents, summaries, output_dir) -> None:
    for language in documents.keys():
        with JsonlWriter(f'{output_dir}/{language}.jsonl') as out:
            for instance_id in sorted(documents[language].keys()):
                lang_documents = documents[language][instance_id]
                lang_summaries = summaries[language][instance_id]
                out.write({
                    'instance_id': instance_id,
                    'documents': lang_documents,
                    'summaries': lang_summaries
                })


def setup(documents_zip: str, summaries_zip: str, output_dir: str) -> None:
    documents = load_documents(documents_zip)
    summaries = load_summaries(summaries_zip)
    save_data(documents, summaries, output_dir)



import argparse
import unittest

from sacrerouge.common.testing.util import command_exists


class TestUtil(unittest.TestCase):
    def test_command_exists(self):
        parser = argparse.ArgumentParser()
        assert command_exists(parser, ['test']) is False

        subparsers = parser.add_subparsers()
        assert command_exists(parser, ['test']) is False

        parser_A = subparsers.add_parser('A')
        parser_B = subparsers.add_parser('B')
        assert command_exists(parser, ['A'])
        assert command_exists(parser, ['B'])
        assert not command_exists(parser, ['C'])
        assert not command_exists(parser, ['B', 'C'])

        subparsers_B = parser_B.add_subparsers()
        parser_C = subparsers_B.add_parser('C')
        assert command_exists(parser, ['B', 'C'])
        assert not command_exists(parser, ['B', 'D'])


from spacy.tokens import Token
from typing import List

from sacrerouge.metrics.decomposed_rouge.categorizers import Categorizer


class NERCategorizer(Categorizer):
    def __init__(self, name: str, tag_sets: List[List[str]]):
        super().__init__(name)
        self.tag_sets = [set(tags) for tags in tag_sets]

    def is_match(self, token1: Token, token2: Token) -> bool:
        for tags in self.tag_sets:
            if token1.ent_type_ in tags and token2.ent_type_ in tags:
                return True
        return False

    def is_candidate(self, token: Token) -> bool:
        for tags in self.tag_sets:
            if token.ent_type_ in tags:
                return True
        return False



import os
import pytest
import unittest
from glob import glob

from sacrerouge.io import JsonlReader

_mds_dir = 'datasets/multiling/multiling2015/mds'


@pytest.mark.skipif(not os.path.exists(_mds_dir), reason='MultiLing 2015 MDS data does not exist')
class TestMultiLing2015MDS(unittest.TestCase):
    def test_sanity_checks(self):
        file_paths = glob(f'{_mds_dir}/??.train.jsonl')
        assert len(file_paths) == 10  # 10 languages

        # Make sure each one has 3 instances with non-empty documents and summaries
        for file_path in file_paths:
            instances = JsonlReader(file_path).read()
            assert len(instances) == 3
            for instance in instances:
                assert len(instance['documents']) > 0
                for document in instance['documents']:
                    assert len(document['text']) > 0
                assert len(instance['summaries']) > 0
                for summary in instance['summaries']:
                    assert len(summary['text']) > 0

        # The test currently does not have summaries
        file_paths = glob(f'{_mds_dir}/??.test.jsonl')
        assert len(file_paths) == 10  # 10 languages

        # Make sure each one has 3 instances with non-empty documents and summaries
        for file_path in file_paths:
            instances = JsonlReader(file_path).read()
            assert len(instances) == 7
            for instance in instances:
                assert len(instance['documents']) > 0
                for document in instance['documents']:
                    assert len(document['text']) > 0
                assert 'summaries' not in instance



import os
import pytest
import unittest
from collections import defaultdict

from sacrerouge.data import Pyramid, PyramidAnnotation
from sacrerouge.io import JsonlReader


_pyramids_file_path = 'datasets/duc-tac/duc2006/v1.0/task1.pyramids.jsonl'
_pyramid_annotations_file_path = 'datasets/duc-tac/duc2006/v1.0/task1.pyramid-annotations.jsonl'


@pytest.mark.skipif(not os.path.exists(_pyramids_file_path), reason='DUC 2006 Pyramids file does not exist')
@pytest.mark.skipif(not os.path.exists(_pyramid_annotations_file_path), reason='DUC 2006 Pyramid annotations file does not exist')
class TestDUC2006Pyramids(unittest.TestCase):
    def test_pyramids(self):
        """Do some basic sanity tests on the files."""
        pyramids = JsonlReader(_pyramids_file_path, Pyramid).read()
        annotations = JsonlReader(_pyramid_annotations_file_path, PyramidAnnotation).read()

        instance_id_to_pyramid = {}
        for pyramid in pyramids:
            instance_id_to_pyramid[pyramid.instance_id] = pyramid

        instance_id_to_annotations = defaultdict(list)
        for annotation in annotations:
            instance_id_to_annotations[annotation.instance_id].append(annotation)

        assert instance_id_to_pyramid.keys() == instance_id_to_annotations.keys()
        for instance_id, pyramid in instance_id_to_pyramid.items():
            assert len(pyramid.summaries) == 4
            assert len(pyramid.summarizer_ids) == 4
            for reference in pyramid.summaries:
                assert len(reference) > 0

            scu_ids = set([scu.scu_id for scu in pyramid.scus])
            for annotation in instance_id_to_annotations[instance_id]:
                assert len(annotation.summary) > 0, (instance_id, annotation.summarizer_id)
                for scu in annotation.scus:
                    assert scu.scu_id in scu_ids, (scu.scu_id, scu_ids)

    def _test_example(self, pyramid: Pyramid, annotation: PyramidAnnotation) -> None:
        scu_to_label = {scu.scu_id: scu.label for scu in pyramid.scus}
        for scu in annotation.scus:
            assert scu.label == scu_to_label[scu.scu_id]

    def test_pyramid_examples(self):
        """Pick some random examples and test them."""
        pyramids = JsonlReader(_pyramids_file_path, Pyramid).read()
        annotations = JsonlReader(_pyramid_annotations_file_path, PyramidAnnotation).read()

        instance_id_to_pyramid = {}
        for pyramid in pyramids:
            instance_id_to_pyramid[pyramid.instance_id] = pyramid

        instance_id_to_annotations = defaultdict(dict)
        for annotation in annotations:
            instance_id_to_annotations[annotation.instance_id][annotation.summarizer_id] = annotation

        pyramid = instance_id_to_pyramid['d0608']
        assert pyramid.summarizer_ids == ['D', 'E', 'H', 'J']
        annotation = instance_id_to_annotations['d0608']['23']
        label = 'side air bags can kill or seriously injure children under 12 and urged manufacturers installing the devices in rear seats to let drivers decide if they want them activated'
        assert len(annotation.scus) == 4
        assert annotation.scus[1].scu_id == 18
        assert len(annotation.scus[1].contributors) == 1
        assert annotation.scus[1].contributors[0].label == label
        assert len(annotation.scus[1].contributors[0].parts) == 1
        assert annotation.scus[1].contributors[0].parts[0].text == label
        start = annotation.scus[1].contributors[0].parts[0].start
        end = annotation.scus[1].contributors[0].parts[0].end
        assert annotation.summary[start:end] == label
        self._test_example(pyramid, annotation)

        pyramid = instance_id_to_pyramid['d0629']
        assert pyramid.summarizer_ids == ['B', 'G', 'H', 'I']
        annotation = instance_id_to_annotations['d0629']['15']
        label = 'Melissa typically enters a computer hidden in a Microsoft Word 97 or Word 2000'
        assert len(annotation.scus) == 3
        assert annotation.scus[1].scu_id == 20
        assert len(annotation.scus[1].contributors) == 2
        assert annotation.scus[1].contributors[0].label == label
        assert len(annotation.scus[1].contributors[0].parts) == 1
        assert annotation.scus[1].contributors[0].parts[0].text == label
        start = annotation.scus[1].contributors[0].parts[0].start
        end = annotation.scus[1].contributors[0].parts[0].end
        assert annotation.summary[start:end] == label
        self._test_example(pyramid, annotation)



import argparse
import functools
import json
import logging
import numpy as np
import os
import random
from overrides import overrides
from scipy.stats import kendalltau, pearsonr, spearmanr
from typing import Dict, List, Tuple, Union

from sacrerouge.commands import RootSubcommand
from sacrerouge.commands.correlate import load_metrics, filter_metrics, merge_metrics
from sacrerouge.common.logging import prepare_global_logging
from sacrerouge.data import Metrics
from sacrerouge.stats import convert_to_matrices, corr_diff_test, global_corr, summary_level_corr, system_level_corr

logger = logging.getLogger(__name__)


def _get_hypotheses(two_tailed: bool,
                    dependent_metric: str,
                    metric_A: str,
                    metric_B: str,) -> Tuple[str, str]:
    if two_tailed:
        return f'r({metric_A}, {dependent_metric}) == r({metric_B}, {dependent_metric})', \
               f'r({metric_A}, {dependent_metric}) != r({metric_B}, {dependent_metric})'
    else:
        return f'r({metric_A}, {dependent_metric}) <= r({metric_B}, {dependent_metric})', \
               f'r({metric_A}, {dependent_metric}) > r({metric_B}, {dependent_metric})'


def _run_test(corr_func,
              X: np.ndarray, Y: np.ndarray, Z: np.ndarray,
              test_method: str,
              alpha: float,
              two_tailed: bool) -> Dict:
    pearson = functools.partial(corr_func, pearsonr)
    spearman = functools.partial(corr_func, spearmanr)
    kendall = functools.partial(corr_func, kendalltau)

    r_pvalue = corr_diff_test(pearson, X, Y, Z, test_method, two_tailed)
    rho_pvalue = corr_diff_test(spearman, X, Y, Z, test_method, two_tailed)
    tau_pvalue = corr_diff_test(kendall, X, Y, Z, test_method, two_tailed)

    # For some reason, without casting `pvalue <= alpha` to a bool, the result
    # would be type `bool_` which was not json serializable
    return {
        'pearson': {
            'pvalue': r_pvalue,
            'is_significant': bool(r_pvalue <= alpha)
        },
        'spearman': {
            'pvalue': rho_pvalue,
            'is_significant': bool(rho_pvalue <= alpha)
        },
        'kendall': {
            'pvalue': tau_pvalue,
            'is_significant': bool(tau_pvalue <= alpha)
        },
    }


def run_hypothesis_tests(metrics_jsonl_files_or_metrics_list: Union[str, List[str], List[Metrics]],
                         dependent_metric: str,
                         metric_A: str,
                         metric_B: str,
                         summarizer_type: str,
                         test_method: str = 'permutation-both',
                         alpha: float = 0.05,
                         two_tailed: bool = True,
                         skip_summary_level: bool = False,
                         skip_system_level: bool = False,
                         skip_global: bool = False) -> Dict:
    if isinstance(metrics_jsonl_files_or_metrics_list, str):
        # A single file
        metrics_list = load_metrics([metrics_jsonl_files_or_metrics_list])
    elif isinstance(metrics_jsonl_files_or_metrics_list, list) and all(
            isinstance(item, str) for item in metrics_jsonl_files_or_metrics_list):
        # A list of files
        metrics_list = load_metrics(metrics_jsonl_files_or_metrics_list)
    else:
        # A list of metrics
        assert isinstance(metrics_jsonl_files_or_metrics_list, list) and all(
            isinstance(item, Metrics) for item in metrics_jsonl_files_or_metrics_list)
        metrics_list = metrics_jsonl_files_or_metrics_list

    # Merge duplicate metrics objects into one
    metrics_list = merge_metrics(metrics_list)

    for metrics in metrics_list:
        metrics.flatten_keys()

    metrics_list = filter_metrics(metrics_list, summarizer_type, dependent_metric, metric_A, metric_B)
    for metrics in metrics_list:
        metrics.select_metrics([dependent_metric, metric_A, metric_B])
        metrics.average_values()

    # Follow the math in the paper: the dependent metric is Z
    X, Y, Z = convert_to_matrices(metrics_list, metric_A, metric_B, dependent_metric)

    H0, H1 = _get_hypotheses(two_tailed, dependent_metric, metric_A, metric_B)
    results = {
        'dependent_metric': dependent_metric,
        'metric_A': metric_A,
        'metric_B': metric_B,
        'summarizer_type': summarizer_type,
        'test_method': test_method,
        'alpha': alpha,
        'two_tailed': two_tailed,
        'H0': H0,
        'H1': H1
    }
    if not skip_summary_level:
        results['summary_level'] = _run_test(summary_level_corr, X, Y, Z, test_method, alpha, two_tailed)

    if not skip_system_level:
        results['system_level'] = _run_test(system_level_corr, X, Y, Z, test_method, alpha, two_tailed)

    if not skip_global:
        results['global'] = _run_test(global_corr, X, Y, Z, test_method, alpha, two_tailed)

    return results


@RootSubcommand.register('stat-sig-test')
class StatisticalSignificanceTestSubcommand(RootSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Run hypothesis testing on the difference between the correlation between metric A and the ' \
                      'dependent metric versus metric B and the dependent metric'
        self.parser = parser.add_parser('stat-sig-test', description=description, help=description)
        self.parser.add_argument(
            '--metrics-jsonl-files',
            nargs='+',
            help='The jsonl files with the metric values. If the values are split across multiple files, they can all '
                 'be passed as arguments.',
            required=True
        )
        self.parser.add_argument(
            '--dependent-metric',
            type=str,
            help='The flattened name of the dependent metric against which A and B will be correlated',
            required=True
        )
        self.parser.add_argument(
            '--metric-A',
            type=str,
            help='The flattened name of metric A',
            required=True
        )
        self.parser.add_argument(
            '--metric-B',
            type=str,
            help='The flattened name of metric B',
            required=True
        )
        self.parser.add_argument(
            '--summarizer-type',
            choices=['all', 'reference', 'peer'],
            help='The type of summarizer which should be included in the correlation calculation',
            required=True
        )
        self.parser.add_argument(
            '--hypothesis-test',
            choices=['bootstrap-system', 'bootstrap-input', 'bootstrap-both', 'permutation-both',
                     'permutation-input', 'permutation-system', 'williams'],
            default='permutation-both',
            help='The hypothesis test to use'
        )
        self.parser.add_argument(
            '--confidence',
            type=float,
            default=95,
            help='The confidence level of the hypothesis test'
        )
        self.parser.add_argument(
            '--num-tails',
            type=int,
            choices=[1, 2],
            default=1,
            help='The number of tails to use in the hypothesis test'
        )
        self.parser.add_argument(
            '--random-seed',
            type=int,
            help='The random seed to use for numpy. Python random will be this number plus one'
        )
        self.parser.add_argument(
            '--skip-summary-level',
            action='store_true',
            help='Indicates the summary-level correlations should not be tested'
        )
        self.parser.add_argument(
            '--skip-system-level',
            action='store_true',
            help='Indicates the system-level correlations should not be tested'
        )
        self.parser.add_argument(
            '--skip-global',
            action='store_true',
            help='Indicates the global correlations should not be tested'
        )
        self.parser.add_argument(
            '--output-file',
            type=str,
            help='The json output file which will contain the test results'
        )
        self.parser.add_argument(
            '--log-file',
            type=str,
            help='The file where the log should be written'
        )
        self.parser.add_argument(
            '--silent',
            action='store_true',
            help='Controls whether the log should be written to stdout'
        )
        self.parser.set_defaults(func=self.run)

    def run(self, args):
        prepare_global_logging(file_path=args.log_file, silent=args.silent)

        if args.random_seed is not None:
            np.random.seed(args.random_seed)
            random.seed(args.random_seed + 1)

        two_tailed = args.num_tails == 2
        alpha = 1.0 - args.confidence / 100
        results = run_hypothesis_tests(args.metrics_jsonl_files,
                                       args.dependent_metric,
                                       args.metric_A,
                                       args.metric_B,
                                       args.summarizer_type,
                                       test_method=args.hypothesis_test,
                                       alpha=alpha,
                                       two_tailed=two_tailed,
                                       skip_summary_level=args.skip_summary_level,
                                       skip_system_level=args.skip_system_level,
                                       skip_global=args.skip_global)

        if args.output_file:
            dirname = os.path.dirname(args.output_file)
            if dirname:
                os.makedirs(dirname, exist_ok=True)
            with open(args.output_file, 'w') as out:
                out.write(json.dumps(results, indent=2))

        if not args.silent:
            logger.info(json.dumps(results, indent=2))


import tarfile
from collections import defaultdict
from nltk.tokenize import sent_tokenize

from sacrerouge.io import JsonlWriter


def load_documents(tar_path: str):
    documents = defaultdict(dict)
    with tarfile.open(tar_path, 'r') as tar:
        for member in tar.getmembers():
            if member.name.startswith('multilingMss2015Eval/body/text') and member.name.endswith('.txt'):
                language = member.name[31:33]
                instance_id = member.name[34:66]
                text = tar.extractfile(member).read().decode().splitlines()
                if language == 'en':
                    text = [sent_tokenize(paragraph) for paragraph in text]
                document = {
                    'filename': instance_id,
                    'text': text
                }
                documents[language][instance_id] = document
    return documents


def load_summaries(tar_path: str):
    summaries = defaultdict(dict)
    with tarfile.open(tar_path, 'r') as tar:
        for member in tar.getmembers():
            if member.name.startswith('multilingMss2015Eval/summary') and member.name.endswith('.txt'):
                language = member.name[29:31]
                instance_id = member.name[32:64]
                text = tar.extractfile(member).read().decode().splitlines()
                if language == 'en':
                    text = [sentence for paragraph in text for sentence in sent_tokenize(paragraph)]
                summary = {
                    'text': text
                }
                summaries[language][instance_id] = summary
    return summaries


def save_data(documents, summaries, output_dir):
    for language in documents.keys():
        with JsonlWriter(f'{output_dir}/{language}.jsonl') as out:
            for filename, document in documents[language].items():
                summary = summaries[language][filename]
                out.write({
                    'instance_id': filename,
                    'document': document,
                    'summary': summary
                })


def setup(tar_path: str, output_dir: str):
    documents = load_documents(tar_path)
    summaries = load_summaries(tar_path)
    save_data(documents, summaries, output_dir)


import json
import os
import subprocess
import pytest
import unittest

from sacrerouge.common import TemporaryDirectory
from sacrerouge.common.testing import MULTILING_METRICS
from sacrerouge.common.testing.util import sacrerouge_command_exists


class TestCorrelate(unittest.TestCase):
    def test_command_exists(self):
        assert sacrerouge_command_exists(['correlate'])

    def test_correlate_reference(self):
        # This is a regression test for the "correlate" command. It does not test if it's accurate
        # TODO This needs to be a better test. There are too few summarization systems to get interesting
        # correlations
        with TemporaryDirectory() as temp_dir:
            command = [
                'python', '-m', 'sacrerouge', 'correlate',
                '--metrics-jsonl-files', MULTILING_METRICS,
                '--metrics', 'rouge-1_jk_precision', 'grade',
                '--summarizer-type', 'reference',
                '--output-file', f'{temp_dir}/correlations.json',
                '--silent'
            ]
            subprocess.run(command, check=True)
            correlations = json.load(open(f'{temp_dir}/correlations.json', 'r'))

            assert correlations['metric1'] == 'rouge-1_jk_precision'
            assert correlations['metric2'] == 'grade'
            assert correlations['summarizer_type'] == 'reference'

            assert correlations['summary_level']['pearson']['r'] == pytest.approx(0.3333, abs=1e-4)
            assert correlations['summary_level']['spearman']['rho'] == pytest.approx(0.3333, abs=1e-4)
            assert correlations['summary_level']['kendall']['tau'] == pytest.approx(0.3333, abs=1e-4)
            assert correlations['summary_level']['num_summary_groups'] == 3

            assert correlations['system_level']['pearson']['r'] == pytest.approx(1.0, abs=1e-4)
            assert correlations['system_level']['spearman']['rho'] == pytest.approx(1.0, abs=1e-4)
            assert correlations['system_level']['kendall']['tau'] == pytest.approx(1.0, abs=1e-4)
            assert correlations['system_level']['num_summarizers'] == 2

            assert correlations['global']['pearson']['r'] == pytest.approx(0.6225273481541307, abs=1e-4)
            assert correlations['global']['spearman']['rho'] == pytest.approx(0.8285714285714287, abs=1e-4)
            assert correlations['global']['kendall']['tau'] == pytest.approx(0.7333333333333333, abs=1e-4)
            assert correlations['global']['num_summaries'] == 6

    def test_correlate_peer(self):
        # This is a regression test for the "correlate" command. It does not test if it's accurate
        # TODO This needs to be a better test. There are too few summarization systems to get interesting
        # correlations
        with TemporaryDirectory() as temp_dir:
            command = [
                'python', '-m', 'sacrerouge', 'correlate',
                '--metrics-jsonl-files', MULTILING_METRICS,
                '--metrics', 'rouge-1_precision', 'grade',
                '--summarizer-type', 'peer',
                '--output-file', f'{temp_dir}/correlations.json',
                '--silent'
            ]
            subprocess.run(command, check=True)
            correlations = json.load(open(f'{temp_dir}/correlations.json', 'r'))

            assert correlations['metric1'] == 'rouge-1_precision'
            assert correlations['metric2'] == 'grade'
            assert correlations['summarizer_type'] == 'peer'

            assert correlations['summary_level']['pearson']['r'] == pytest.approx(-0.3333, abs=1e-4)
            assert correlations['summary_level']['spearman']['rho'] == pytest.approx(-0.3333, abs=1e-4)
            assert correlations['summary_level']['kendall']['tau'] == pytest.approx(-0.3333, abs=1e-4)
            assert correlations['summary_level']['num_summary_groups'] == 3

            assert correlations['system_level']['pearson']['r'] == pytest.approx(-1.0, abs=1e-4)
            assert correlations['system_level']['spearman']['rho'] == pytest.approx(-1.0, abs=1e-4)
            assert correlations['system_level']['kendall']['tau'] == pytest.approx(-1.0, abs=1e-4)
            assert correlations['system_level']['num_summarizers'] == 2

            assert correlations['global']['pearson']['r'] == pytest.approx(-0.33056857901135617, abs=1e-4)
            assert correlations['global']['spearman']['rho'] == pytest.approx(-0.3768511731740915, abs=1e-4)
            assert correlations['global']['kendall']['tau'] == pytest.approx(-0.27602622373694163, abs=1e-4)
            assert correlations['global']['num_summaries'] == 6

    def test_correlate_all(self):
        # This is a regression test for the "correlate" command. It does not test if it's accurate
        with TemporaryDirectory() as temp_dir:
            command = [
                'python', '-m', 'sacrerouge', 'correlate',
                '--metrics-jsonl-files', MULTILING_METRICS,
                '--metrics', 'rouge-1_jk_precision', 'grade',
                '--summarizer-type', 'all',
                '--output-file', f'{temp_dir}/correlations.json',
                '--silent'
            ]
            subprocess.run(command, check=True)
            correlations = json.load(open(f'{temp_dir}/correlations.json', 'r'))

            assert correlations['metric1'] == 'rouge-1_jk_precision'
            assert correlations['metric2'] == 'grade'
            assert correlations['summarizer_type'] == 'all'

            assert correlations['summary_level']['pearson']['r'] == pytest.approx(0.4365526945989437, abs=1e-4)
            assert correlations['summary_level']['spearman']['rho'] == pytest.approx(0.3720759220056127, abs=1e-4)
            assert correlations['summary_level']['kendall']['tau'] == pytest.approx(0.1719691730561296, abs=1e-4)
            assert correlations['summary_level']['num_summary_groups'] == 3

            assert correlations['system_level']['pearson']['r'] == pytest.approx(0.28732601225892834, abs=1e-4)
            assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.19999999999999998, abs=1e-4)
            assert correlations['system_level']['kendall']['tau'] == pytest.approx(0.0, abs=1e-4)
            assert correlations['system_level']['num_summarizers'] == 4

            assert correlations['global']['pearson']['r'] == pytest.approx(0.34183806349510004, abs=1e-4)
            assert correlations['global']['spearman']['rho'] == pytest.approx(0.4035707976004214, abs=1e-4)
            assert correlations['global']['kendall']['tau'] == pytest.approx(0.28603877677367767, abs=1e-4)
            assert correlations['global']['num_summaries'] == 12

    def test_all_summary_level_correlations(self):
        # This is a regression test for the "correlate" command. It does not test if it's accurate
        with TemporaryDirectory() as temp_dir:
            command = [
                'python', '-m', 'sacrerouge', 'correlate',
                '--metrics-jsonl-files', MULTILING_METRICS,
                '--metrics', 'rouge-1_jk_precision', 'grade',
                '--summarizer-type', 'all',
                '--output-file', f'{temp_dir}/correlations.json',
                '--silent'
            ]
            subprocess.run(command, check=True)

            # Check the original correlations
            correlations = json.load(open(f'{temp_dir}/correlations.json', 'r'))

            assert correlations['metric1'] == 'rouge-1_jk_precision'
            assert correlations['metric2'] == 'grade'
            assert correlations['summarizer_type'] == 'all'

            assert correlations['summary_level']['pearson']['r'] == pytest.approx(0.4365526945989437, abs=1e-4)
            assert correlations['summary_level']['spearman']['rho'] == pytest.approx(0.3720759220056127, abs=1e-4)
            assert correlations['summary_level']['kendall']['tau'] == pytest.approx(0.1719691730561296, abs=1e-4)
            assert correlations['summary_level']['num_summary_groups'] == 3

            assert correlations['system_level']['pearson']['r'] == pytest.approx(0.28732601225892834, abs=1e-4)
            assert correlations['system_level']['spearman']['rho'] == pytest.approx(0.19999999999999998, abs=1e-4)
            assert correlations['system_level']['kendall']['tau'] == pytest.approx(0.0, abs=1e-4)
            assert correlations['system_level']['num_summarizers'] == 4

            assert correlations['global']['pearson']['r'] == pytest.approx(0.34183806349510004, abs=1e-4)
            assert correlations['global']['spearman']['rho'] == pytest.approx(0.4035707976004214, abs=1e-4)
            assert correlations['global']['kendall']['tau'] == pytest.approx(0.28603877677367767, abs=1e-4)
            assert correlations['global']['num_summaries'] == 12

    def test_skip_calculations(self):
        # Ensures the flags to skip calculating specific correlations work
        with TemporaryDirectory() as temp_dir:
            command = [
                'python', '-m', 'sacrerouge', 'correlate',
                '--metrics-jsonl-files', MULTILING_METRICS,
                '--metrics', 'rouge-1_jk_precision', 'grade',
                '--summarizer-type', 'all',
                '--output-file', f'{temp_dir}/correlations.json',
                '--silent',
                '--skip-summary-level'
            ]
            subprocess.run(command, check=True)
            correlations = json.load(open(f'{temp_dir}/correlations.json', 'r'))
            assert 'summary_level' not in correlations

            command = [
                'python', '-m', 'sacrerouge', 'correlate',
                '--metrics-jsonl-files', MULTILING_METRICS,
                '--metrics', 'rouge-1_jk_precision', 'grade',
                '--summarizer-type', 'all',
                '--output-file', f'{temp_dir}/correlations.json',
                '--silent',
                '--skip-system-level'
            ]
            subprocess.run(command, check=True)
            correlations = json.load(open(f'{temp_dir}/correlations.json', 'r'))
            assert 'system_level' not in correlations

            command = [
                'python', '-m', 'sacrerouge', 'correlate',
                '--metrics-jsonl-files', MULTILING_METRICS,
                '--metrics', 'rouge-1_jk_precision', 'grade',
                '--summarizer-type', 'all',
                '--output-file', f'{temp_dir}/correlations.json',
                '--silent',
                '--skip-global'
            ]
            subprocess.run(command, check=True)
            correlations = json.load(open(f'{temp_dir}/correlations.json', 'r'))
            assert 'global' not in correlations

    def test_plots(self):
        # Tests to ensure plot files exist
        with TemporaryDirectory() as temp_dir:
            system_plot_file = f'{temp_dir}/system.pdf'
            global_plot_file = f'{temp_dir}/global.pdf'
            command = [
                'python', '-m', 'sacrerouge', 'correlate',
                '--metrics-jsonl-files', MULTILING_METRICS,
                '--metrics', 'rouge-1_jk_precision', 'grade',
                '--summarizer-type', 'all',
                '--output-file', f'{temp_dir}/correlations.json',
                '--silent',
                '--system-level-output-plot', system_plot_file,
                '--global-output-plot', global_plot_file
            ]
            subprocess.run(command, check=True)
            assert os.path.exists(system_plot_file)
            assert os.path.exists(global_plot_file)



import os
import pytest
import unittest
from glob import glob

from sacrerouge.io import JsonlReader

_sds_dir = 'datasets/multiling/multiling2013/sds'


@pytest.mark.skipif(not os.path.exists(_sds_dir), reason='MultiLing 2013 SDS data does not exist')
class TestMultiLing2013SDS(unittest.TestCase):
    def test_sanity_checks(self):
        file_paths = glob(f'{_sds_dir}/*.jsonl')
        assert len(file_paths) == 40  # 40 languages

        # Make sure each one has 30 instances with non-empty documents and summaries
        for file_path in file_paths:
            instances = JsonlReader(file_path).read()
            assert len(instances) == 30
            for instance in instances:
                assert len(instance['document']) > 0
                assert len(instance['summary']) > 0


import unittest

from sacrerouge.data.dataset_readers.document_based import flatten_document


class TestDocumentBasedReader(unittest.TestCase):
    def test_flatten_document(self):
        document = [
            'A',
            ['B', ['C', 'D', ['E']]],
            ['F']
        ]
        assert flatten_document(document) == ['A', 'B', 'C', 'D', 'E', 'F']



import argparse
from overrides import overrides

from sacrerouge.datasets.multiling.multiling2013 import mds, mds_metrics, sds
from sacrerouge.commands import DatasetSetupSubcommand
from sacrerouge.common.util import download_file_from_google_drive


@DatasetSetupSubcommand.register('multiling2013')
class MultiLing2013Subcommand(DatasetSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the MultiLing 2013 dataset'
        self.parser = parser.add_parser('multiling2013', description=description, help=description)
        self.parser.add_argument(
            'output_dir',
            type=str,
            help='The directory where the data should be saved'
        )
        self.parser.add_argument(
            '--mds-documents-zip',
            type=str,
            help='The path to "SourceTextsV2b.zip"'
        )
        self.parser.add_argument(
            '--mds-model-summaries-zip',
            type=str,
            help='The path to "ModelSummaries- 20130605.zip"'
        )
        self.parser.add_argument(
            '--mds-peer-summaries-zip',
            type=str,
            help='The path to "PeerSummaries - 20130601.zip"'
        )
        self.parser.add_argument(
            '--metrics-zip',
            type=str,
            help='The path to "MultiDocument-ManualGrades-20130717.zip"'
        )
        self.parser.set_defaults(subfunc=self.run)

    def _notify_about_license(self):
        print('This dataset is distributed by MultiLing 2013. If you use this dataset, please refer to their '
              'website (http://multiling.iit.demokritos.gr/pages/view/662/multiling-2013) '
              'for instructions on proper attribution.')
        print()
        response = input('Continue? [y/n]: ')
        if response.lower() != 'y':
            exit()

    @overrides
    def run(self, args):
        self._notify_about_license()

        # Download the dataset so each individual setup does not need
        # to repeat this. The dataset comes from
        # http://multiling.iit.demokritos.gr/pages/view/1571/datasets
        tar_path = f'{args.output_dir}/multilingpilot2013.tar.bz2'
        download_file_from_google_drive('0B31rakzMfTMZRTZiM29UR3VxYmc', tar_path)
        sds.setup(tar_path, f'{args.output_dir}/sds')

        # MDS data is password protected, so the user must provide the data
        if not all([args.mds_documents_zip, args.mds_model_summaries_zip]):
            print('Skipping setting up MDS task because either documents or model summaries zip not provided')
        else:
            mds.setup(args.mds_documents_zip, args.mds_model_summaries_zip, f'{args.output_dir}/mds')

        if not all([args.mds_model_summaries_zip, args.mds_peer_summaries_zip, args.metrics_zip]):
            print('Skipping setting up MDS metrics because either model/peer summaries or metrics zips not provided')
        else:
            mds_metrics.setup(args.mds_model_summaries_zip, args.mds_peer_summaries_zip, args.metrics_zip, f'{args.output_dir}/mds')



import argparse
import logging
import os
import shutil
from glob import glob
from overrides import overrides
from subprocess import Popen, PIPE
from typing import Dict, List, Tuple

from sacrerouge.commands import MetricSetupSubcommand
from sacrerouge.common import DATA_ROOT, TemporaryDirectory
from sacrerouge.data import MetricsDict
from sacrerouge.data.types import ReferenceType, SummaryType
from sacrerouge.metrics import Metric, ReferenceBasedMetric

logger = logging.getLogger(__name__)


@Metric.register('pyreval')
class PyrEval(ReferenceBasedMetric):
    def __init__(self,
                 environment_name: str = None,
                 pyreval_root: str = f'{DATA_ROOT}/metrics/PyrEval',
                 verbose: bool = False):
        super().__init__()
        self.environment_name = environment_name
        self.pyreval_root = pyreval_root
        self.verbose = verbose

        if self.environment_name is not None:
            if 'CONDA_INIT' not in os.environ:
                raise Exception('If `environment_name` is not none, environment variable "CONDA_INIT" must be set to the path to "conda.sh"')

    def _clean_directories(self):
        # The PyrEval code runs all of its processing in place, so we need to clean up all of its
        # directories in case there is something left over from an old run, which might cause a problem
        # for the next run.
        #
        # This code is based on the clean function in "pyreval.py". The original code will crash if one of
        # the directories it searches for does not exist, so we reimplemented it here.

        # Clean the raw data (not included in the original code)
        for file_path in glob(f'{self.pyreval_root}/Raw/peers/*'):
            if os.path.isfile(file_path):
                os.remove(file_path)
        for file_path in glob(f'{self.pyreval_root}/Raw/model/*'):
            if os.path.isfile(file_path):
                os.remove(file_path)

        # Clean the sentence splits
        for file_path in glob(f'{self.pyreval_root}/Raw/peers/split/*'):
            os.remove(file_path)
        for file_path in glob(f'{self.pyreval_root}/Raw/model/split/*'):
            os.remove(file_path)

        # Clean the preprocessed folder, first the files, then the folders
        for file_path in glob(f'{self.pyreval_root}/Preprocess/peer_summaries/*.xml'):
            os.remove(file_path)
        for dir_path in glob(f'{self.pyreval_root}/Preprocess/peer_summaries/*'):
            shutil.rmtree(dir_path)

        for file_path in glob(f'{self.pyreval_root}/Preprocess/wise_crowd_summaries/*.xml'):
            os.remove(file_path)
        for dir_path in glob(f'{self.pyreval_root}/Preprocess/wise_crowd_summaries/*'):
            shutil.rmtree(dir_path)

        # Pyramid scores
        scores_file = f'{self.pyreval_root}/Pyramid/scores.txt'
        if os.path.exists(scores_file):
            os.remove(scores_file)

        # Pyramids
        for file_path in glob(f'{self.pyreval_root}/Scoring/pyrs/pyramids/*.pyr'):
            os.remove(file_path)

        # SCUs
        for file_path in glob(f'{self.pyreval_root}/Scoring/scu/*.pyr'):
            os.remove(file_path)

        # Sizes
        for file_path in glob(f'{self.pyreval_root}/Scoring/sizes/*.size'):
            os.remove(file_path)

        # Temp
        for file_path in glob(f'{self.pyreval_root}/Scoring/temp/*'):
            os.remove(file_path)

        # The results file
        results_file = f'{self.pyreval_root}/results.csv'
        if os.path.exists(results_file):
            os.remove(results_file)

    def _assert_has_multiple_references(self, references_list: List[List[ReferenceType]]) -> None:
        # PyrEval cannot handle summaries with single references. See https://github.com/serenayj/PyrEval/issues/11
        for references in references_list:
            if len(references) == 1:
                raise Exception(f'PyrEval cannot be run with one reference summary. '
                                f'See https://github.com/serenayj/PyrEval/issues/11 for more details')

    def _flatten_summaries(self, summaries_list: List[List[SummaryType]]) -> List[List[str]]:
        flat_list = []
        for summaries in summaries_list:
            flat_list.append([])
            for summary in summaries:
                if isinstance(summary, list):
                    summary = ' '.join(summary)
                flat_list[-1].append(summary)
        return flat_list

    def _index_summaries(self,
                         summaries_list: List[List[str]],
                         references_list: List[List[str]]) -> Tuple[List[str], Dict[str, int]]:
        index_to_summary = []
        summary_to_index = {}
        for summs_list in [summaries_list, references_list]:
            for summaries in summs_list:
                for summary in summaries:
                    if summary not in summary_to_index:
                        index_to_summary.append(summary)
                        summary_to_index[summary] = len(summary_to_index)
        return index_to_summary, summary_to_index

    def _save_summaries(self, summaries: List[str], output_dir: str) -> None:
        os.makedirs(output_dir, exist_ok=True)
        for i, summary in enumerate(summaries):
            with open(f'{output_dir}/{i}', 'w') as out:
                out.write(summary)

    def _run_through_preprocessing(self) -> None:
        logging.info('Running PyrEval through preprocessing')

        # Each of the steps must be done manually, otherwise the "pyreval.py" code would try to run the preprocessing
        # on both the peer and the model directories. We only want it to do the peer directory.
        #
        # Sentence splitting
        commands = [f'cd {self.pyreval_root}']
        if self.environment_name is not None:
            commands.append(f'source {os.environ["CONDA_INIT"]}')
            commands.append(f'conda activate {self.environment_name}')
        commands.append(f'python2.7 split-sent.py {self.pyreval_root}/Raw/peers {self.pyreval_root}/Raw/peers/split')
        command = ' && '.join(commands)

        logger.info(f'Running command: "{command}"')
        redirect = None if self.verbose else PIPE
        process = Popen(command, stdout=redirect, stderr=redirect, shell=True)
        process.communicate()

        # Stanford
        commands = [f'cd {self.pyreval_root}/Stanford']
        if self.environment_name is not None:
            commands.append(f'source {os.environ["CONDA_INIT"]}')
            commands.append(f'conda activate {self.environment_name}')
        commands.append(f'python2.7 stanford.py {self.pyreval_root}/Raw/peers/split 1 ..')
        command = ' && '.join(commands)

        logger.info(f'Running command: "{command}"')
        redirect = None if self.verbose else PIPE
        process = Popen(command, stdout=redirect, stderr=redirect, shell=True)
        process.communicate()

        # Preprocess
        commands = [f'cd {self.pyreval_root}/Preprocess']
        if self.environment_name is not None:
            commands.append(f'source {os.environ["CONDA_INIT"]}')
            commands.append(f'conda activate {self.environment_name}')
        commands.append(f'python2.7 preprocess.py 1')
        command = ' && '.join(commands)

        logger.info(f'Running command: "{command}"')
        redirect = None if self.verbose else PIPE
        process = Popen(command, stdout=redirect, stderr=redirect, shell=True)
        process.communicate()

        logging.info('Finished preprocessing')

    def _map_file_index_to_directory(self, input_dir: str) -> Dict[int, int]:
        # Each directory has a file "x.ls" where "x" is the summary index
        mapping = {}
        for path in glob(f'{input_dir}/*/*.ls'):
            filename = os.path.basename(path)
            file_index = int(os.path.splitext(filename)[0])
            dir_index = int(os.path.basename(os.path.dirname(path)))
            mapping[file_index] = dir_index
        return mapping

    def _move_summaries_to_temp_dir(self, temp_dir: str):
        os.makedirs(f'{temp_dir}/peers', exist_ok=True)
        for path in glob(f'{self.pyreval_root}/Preprocess/peer_summaries/*'):
            shutil.move(path, f'{temp_dir}/peers/')

    def _copy_file_and_change_id(self, src: str, tgt: str, new_id: int):
        # The processing will fail if the document id isn't aligned with the directory the file is in. The document id
        # is the first entry in each line of these files, where each column is separated with &
        with open(tgt, 'w') as out:
            with open(src, 'r') as f:
                for line in f:
                    columns = line.strip().split('&')
                    columns[0] = str(new_id)
                    out.write('&'.join(columns) + '\n')

    def _copy_summaries_for_processing(self,
                                       summaries: List[str],
                                       summary_to_index: Dict[str, int],
                                       file_index_to_dir_index: Dict[int, int],
                                       input_dir: str,
                                       output_dir: str,
                                       sort_summaries: bool,
                                       unique_summaries: bool) -> List[int]:
        os.makedirs(output_dir, exist_ok=True)

        # The PyrEval code changes its output based on the order of the references, so we sort if necessary
        # to keep the results deterministic
        if sort_summaries:
            summaries = sorted(summaries)

        # Copy the summaries from the temporary directory over to the PyrEval directory. The target directories
        # must be 1, 2, ...
        array_index_to_tgt_index = []
        summary_to_tgt_index = {}
        index = 0
        for summary in summaries:
            if unique_summaries:
                if summary in summary_to_tgt_index:
                    array_index_to_tgt_index.append(summary_to_tgt_index[summary])
                    continue
                else:
                    summary_to_tgt_index[summary] = index
                    array_index_to_tgt_index.append(index)
            else:
                array_index_to_tgt_index.append(index)

            file_index = summary_to_index[summary]
            src = f'{input_dir}/{file_index}.xml'
            tgt = f'{output_dir}/{index}.xml'
            logging.info(f'Copying {src} to {tgt}')
            shutil.copy(f'{src}', f'{tgt}')

            dir_index = file_index_to_dir_index[file_index]
            src_dir = f'{input_dir}/{dir_index}'
            tgt_dir = f'{output_dir}/{index + 1}'
            os.makedirs(tgt_dir)
            for ext in ['.ls', '.segs']:
                src = f'{src_dir}/{file_index}{ext}'
                tgt = f'{tgt_dir}/{index}{ext}'
                logging.info(f'Copying {src} to {tgt}, updating document ID to {index + 1}')
                self._copy_file_and_change_id(src, tgt, index + 1)

            index += 1
        return array_index_to_tgt_index

    def _score_summaries(self, array_index_to_tgt_index: List[int]) -> List[MetricsDict]:
        logging.info('Building pyramids and scoring peers')

        # Each step can be run by piping its ID into the pyreval.py program.
        #   4: pyramid
        #   5 -t: score (-t means to write the results to file)
        for args in ['4', '5 -t']:
            commands = [f'cd {self.pyreval_root}']
            if self.environment_name is not None:
                commands.append(f'source {os.environ["CONDA_INIT"]}')
                commands.append(f'conda activate {self.environment_name}')
            commands.append(f'echo {args} | python2.7 pyreval.py')
            command = ' && '.join(commands)

            logger.info(f'Running command: "{command}"')
            redirect = None if self.verbose else PIPE
            process = Popen(command, stdout=redirect, stderr=redirect, shell=True)
            process.communicate()

        # Parse the results
        results_path = f'{self.pyreval_root}/results.csv'
        if not os.path.exists(results_path):
            raise Exception(f'PyrEval results file does not exist: "{results_path}"')

        # First line is the name of the pyramid
        # Second line is the header
        lines = open(results_path, 'r').read().splitlines()
        metrics_dicts = {}
        for line in lines[2:]:
            index, raw, quality, coverage, comprehensive = line.split(',')
            metrics_dicts[int(index)] = MetricsDict({
                'pyreval': {
                    'raw': int(raw),
                    'quality': float(quality),
                    'coverage': float(coverage),
                    'comprehensive': float(comprehensive),
                }
            })

        metrics_list = []
        for index in array_index_to_tgt_index:
            metrics_list.append(metrics_dicts[index])

        logging.info('Finished building pyramids and scoring peers')
        return metrics_list

    def score_multi_all(self,
                        summaries_list: List[List[SummaryType]],
                        references_list: List[List[ReferenceType]]) -> List[List[MetricsDict]]:
        self._assert_has_multiple_references(references_list)

        # The original code for PyrEval processes exactly 1 pyramid at a time. Therefore, the whole pipeline needs
        # to be run once per item in `references_list`. Each execution of the pipeline will load the Stanford CoreNLP
        # models and run them over the text. Loading the models takes a lot of time, and the preprocessing of
        # the same summary may run multiple times (for instance in jackknifing).
        #
        # To save time, our implementation passes all of the unique peer and reference summaries through the
        # preprocessing step of the pipeline at once, then runs the analysis step per-pyramid afterward. This
        # significantly increases the speed of the processing.

        # Identify the unique summaries so less preprocessing needs to be done
        summaries_list = self._flatten_summaries(summaries_list)
        references_list = self._flatten_summaries(references_list)

        all_summaries, summary_to_index = self._index_summaries(summaries_list, references_list)

        with TemporaryDirectory() as temp_dir:
            # First, clear the PyrEval directory in case the last run was messed up
            self._clean_directories()

            # All of the summaries are saved in the "peers" folder, even if they are references. The PyrEval code
            # normally runs separate steps to process the peer and model directories, which is slower because it requires
            # loading the Stanford models twice, but the preprocessing is the same.
            self._save_summaries(all_summaries, f'{self.pyreval_root}/Raw/peers')

            self._run_through_preprocessing()

            # The PyrEval code will create an xml for summary i called i.xml and a directory with more data for
            # that file. The directory names aren't consistent because they're created by enumerating glob results
            # (which I think are not always deterministically sorted, or I don't want to rely on the assumption
            # that they are sorted). So we have to figure out the mapping from the summary index to the directory
            file_index_to_dir = self._map_file_index_to_directory(f'{self.pyreval_root}/Preprocess/peer_summaries')

            # All of the preprocessed summaries are now moved out of the PyrEval directory (or else they would be
            # used in the rest of the processing) to a temporary directory
            self._move_summaries_to_temp_dir(temp_dir)

            # Remove any extra data which could interfere with processing
            self._clean_directories()

            # Now build the pyramids and score
            metrics_dict_lists = []
            for i, (summaries, references) in enumerate(zip(summaries_list, references_list)):
                array_index_to_tgt_index = self._copy_summaries_for_processing(summaries,
                                                                               summary_to_index,
                                                                               file_index_to_dir,
                                                                               f'{temp_dir}/peers',
                                                                               f'{self.pyreval_root}/Preprocess/peer_summaries',
                                                                               False,
                                                                               True)
                self._copy_summaries_for_processing(references,
                                                    summary_to_index,
                                                    file_index_to_dir,
                                                    f'{temp_dir}/peers',
                                                    f'{self.pyreval_root}/Preprocess/wise_crowd_summaries',
                                                    True,
                                                    False)

                metrics_list = self._score_summaries(array_index_to_tgt_index)
                metrics_dict_lists.append(metrics_list)

                # Clean for the next iteration
                self._clean_directories()

            return metrics_dict_lists


@MetricSetupSubcommand.register('pyreval')
class PyrEvalSetupSubcommand(MetricSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the PyrEval metric'
        self.parser = parser.add_parser('pyreval', description=description, help=description)
        self.parser.add_argument('--force', action='store_true', help='Force setting up the metric again')
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        if args.force and os.path.exists(f'{DATA_ROOT}/metrics/PyrEval'):
            shutil.rmtree(f'{DATA_ROOT}/metrics/PyrEval')

        commands = [
            f'mkdir -p {DATA_ROOT}/metrics',
            f'cd {DATA_ROOT}/metrics',
            f'git clone https://github.com/danieldeutsch/PyrEval',
            f'cd PyrEval',
            f'wget http://nlp.stanford.edu/software/stanford-corenlp-full-2018-10-05.zip',
            f'unzip stanford-corenlp-full-2018-10-05.zip',
            f'mv stanford-corenlp-full-2018-10-05/* Stanford/',
            f'rm stanford-corenlp-full-2018-10-05.zip'
        ]
        command = ' && '.join(commands)

        process = Popen(command, shell=True)
        process.communicate()

        if process.returncode == 0:
            print('PyrEval setup success')
        else:
            print('PyrEval setup failure')



import unittest

from sacrerouge.common.testing.metric_test_cases import ReferenceBasedMetricTestCase
from sacrerouge.common.testing.util import sacrerouge_command_exists
from sacrerouge.metrics import BertScore
from sacrerouge.metrics.bertscore import BERTSCORE_INSTALLED


@unittest.skipIf(not BERTSCORE_INSTALLED, '"bert_score" not installed')
class TestBertScore(ReferenceBasedMetricTestCase):
    def test_bertscore(self):
        # This is a regression test, not necessarily a test for correctness
        metric = BertScore()
        expected_output = [
            {'bertscore': {'precision': 0.8534530401229858, 'recall': 0.8503388166427612, 'f1': 0.8518930673599243}},
            {'bertscore': {'precision': 0.8642909526824951, 'recall': 0.8720692992210388, 'f1': 0.8681626319885254}},
            {'bertscore': {'precision': 0.8623127341270447, 'recall': 0.8670819997787476, 'f1': 0.8646908402442932}},
            {'bertscore': {'precision': 0.8549671173095703, 'recall': 0.856684148311615, 'f1': 0.8558247685432434}},
            {'bertscore': {'precision': 0.8397505283355713, 'recall': 0.8507593870162964, 'f1': 0.8451763987541199}},
            {'bertscore': {'precision': 0.8410190343856812, 'recall': 0.8510528206825256, 'f1': 0.8460061550140381}},
            {'bertscore': {'precision': 0.8653707504272461, 'recall': 0.8665218949317932, 'f1': 0.8659459948539734}},
            {'bertscore': {'precision': 0.868201494216919, 'recall': 0.8704793453216553, 'f1': 0.8693389296531677}},
            {'bertscore': {'precision': 0.9007625579833984, 'recall': 0.8653964400291443, 'f1': 0.8827253580093384}},
            {'bertscore': {'precision': 0.8833473920822144, 'recall': 0.8772095441818237, 'f1': 0.8802677989006042}},
            {'bertscore': {'precision': 0.8683465719223022, 'recall': 0.8797310590744019, 'f1': 0.8740017414093018}},
            {'bertscore': {'precision': 0.8667279481887817, 'recall': 0.8681085109710693, 'f1': 0.8665944337844849}}
        ]
        super().assert_expected_output(metric, expected_output)

    def test_bertscore_order_invariant(self):
        metric = BertScore()
        self.assert_order_invariant(metric)

    def test_command_exists(self):
        assert sacrerouge_command_exists(['bertscore'])

    def test_setup_command_exists(self):
        assert sacrerouge_command_exists(['setup-metric', 'bertscore'])


import argparse
import json
import logging
import os
from overrides import overrides
from subprocess import Popen, PIPE
from typing import List

from sacrerouge.commands import MetricSetupSubcommand
from sacrerouge.common import DATA_ROOT, TemporaryDirectory
from sacrerouge.io import JsonlWriter
from sacrerouge.data import MetricsDict
from sacrerouge.data.types import SummaryType
from sacrerouge.metrics import Metric, ReferenceFreeMetric

logger = logging.getLogger(__name__)


@Metric.register('sum-qe')
class SumQE(ReferenceFreeMetric):
    def __init__(self,
                 model_file: str = f'{DATA_ROOT}/metrics/SumQE/models/multitask_5-duc2006_duc2007.npy',
                 sum_qe_root: str = f'{DATA_ROOT}/metrics/SumQE',
                 python_binary: str = 'python',
                 verbose: bool = False):
        super().__init__()
        self.model_file = os.path.abspath(model_file)
        self.sum_qe_root = sum_qe_root
        self.python_binary = python_binary
        self.verbose = verbose

        if not os.path.exists(model_file):
            raise Exception(f'Path "{model_file}" does not exist. Have you setup SumQE?')
        if not os.path.exists(sum_qe_root):
            raise Exception(f'Path "{sum_qe_root}" does not exist. Have you setup SumQE?')

    def _flatten_summary(self, summary: SummaryType) -> str:
        if isinstance(summary, list):
            return ' '.join(summary)
        return summary

    def _run(self, summaries_list: List[List[SummaryType]]) -> List[List[MetricsDict]]:
        with TemporaryDirectory() as temp_dir:
            summaries_file = f'{temp_dir}/summaries.jsonl'
            predictions_file = f'{temp_dir}/predictions.json'

            # Save all of the summaries to a file, keeping track of the indices
            # that are empty summaries
            empty_summaries = set()
            with JsonlWriter(summaries_file) as out:
                index = 0
                for summaries in summaries_list:
                    for summary in summaries:
                        summary = self._flatten_summary(summary)
                        if len(summary) > 0:
                            out.write({'summary': summary})
                        else:
                            empty_summaries.add(index)
                        index += 1

            commands = [
                f'cd {self.sum_qe_root}',
                ' '.join([
                    self.python_binary, '-m', 'src.BERT_experiments.predict',
                    summaries_file,
                    self.model_file,
                    predictions_file
                ])
            ]
            command = ' && '.join(commands)

            logger.info(f'Running SumQE command: "{command}"')
            redirect = None if self.verbose else PIPE
            process = Popen(command, stdout=redirect, stderr=redirect, shell=True)
            stdout, stderr = process.communicate()

            predictions = json.loads(open(predictions_file, 'r').read())

            index = 0
            output_index = 0
            metrics_lists = []
            for summaries in summaries_list:
                metrics_lists.append([])
                for _ in summaries:
                    if index in empty_summaries:
                        metrics_lists[-1].append(MetricsDict({
                            'SumQE': {
                                'Q1': 0.0,
                                'Q2': 0.0,
                                'Q3': 0.0,
                                'Q4': 0.0,
                                'Q5': 0.0
                            }
                        }))
                    else:
                        preds = predictions[output_index]
                        metrics_lists[-1].append(MetricsDict({
                            'SumQE': {
                                'Q1': preds[0],
                                'Q2': preds[1],
                                'Q3': preds[2],
                                'Q4': preds[3],
                                'Q5': preds[4]
                            }
                        }))
                        output_index += 1
                    index += 1

            return metrics_lists

    def score_multi_all(self, summaries_list: List[List[SummaryType]]) -> List[List[MetricsDict]]:
        return self._run(summaries_list)


@MetricSetupSubcommand.register('sum-qe')
class SumQESetupSubcommand(MetricSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the Sum-QE metric'
        self.parser = parser.add_parser('sum-qe', description=description, help=description)
        self.parser.add_argument(
            '--download-2005-2006-model',
            action='store_true',
            help='Indicates that the model trained on DUC 2005 and 2006 should be downloaded'
        )
        self.parser.add_argument(
            '--download-2005-2007-model',
            action='store_true',
            help='Indicates that the model trained on DUC 2005 and 2007 should be downloaded'
        )
        self.parser.add_argument(
            '--download-2006-2007-model',
            action='store_true',
            help='Indicates that the model trained on DUC 2006 and 2007 should be downloaded'
        )
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        commands = [
            f'mkdir -p {DATA_ROOT}/metrics',
            f'cd {DATA_ROOT}/metrics',
            f'git clone https://github.com/danieldeutsch/SumQE',
            f'mkdir -p SumQE/models'
        ]
        if args.download_2005_2006_model:
            commands.append('wget https://danieldeutsch.s3.amazonaws.com/sacrerouge/metrics/SumQE/models/multitask_5-duc2005_duc2006.npy -O SumQE/models/multitask_5-duc2005_duc2006.npy')
        if args.download_2005_2007_model:
            commands.append('wget https://danieldeutsch.s3.amazonaws.com/sacrerouge/metrics/SumQE/models/multitask_5-duc2005_duc2007.npy -O SumQE/models/multitask_5-duc2005_duc2007.npy')
        if args.download_2006_2007_model:
            commands.append('wget https://danieldeutsch.s3.amazonaws.com/sacrerouge/metrics/SumQE/models/multitask_5-duc2006_duc2007.npy -O SumQE/models/multitask_5-duc2006_duc2007.npy')

        command = ' && '.join(commands)

        process = Popen(command, shell=True)
        process.communicate()

        if process.returncode == 0:
            print('SumQE setup success')
        else:
            print('SumQE setup failure')
